<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xml="http://www.w3.org/XML/1998/namespace"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:uuid="java:java.util.UUID" xmlns:xmi="http://schema.omg.org/spec/XMI/2.1"
    xmlns:uml="http://schema.omg.org/spec/UML/2.1.2"
    xmlns:as-xsl="http://www.oasis-open.org/committees/cam/xslt"
    xmlns:UML="org.omg.xmi.namespace.UML"
    exclude-result-prefixes="xs xsd xd uuid xmi uml UML as as-xsl look camed" version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> Mar 25, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> DRRW</xd:p>
            <xd:p/>
        </xd:desc>
    </xd:doc>
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, March, 2011 -->
    <!--
        Purpose  : Utility for use to create models of CAM template structures. Parses input CAM template file and generates model diagram of components.
        Inputs: CAM template.
        Original : 03/25/2011
        Initials : drrw
        Description : Parses input CAM template file and generates model format equivalent
        Uses default CAM template format, including any extensions.
        W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
        W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
        W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
        Updated  - Inits - Change
        03/25/11   drrw   Initial release 
        04/08/11   drrw   Add XMI/UML output
        06/19/11   drrw   Complete xml v2.1 output feature
        06/19/11   drrw   Add ArgoUML output feature
        03/02/12   drrw   Add MagicDraw support
        03/04/12   drrw   Update ownedAttribute output logic for Package to be documentation note instead
        03/05/12   drrw   Remove Altova specific output since Altova will import the regular XMI OK
        03/06/12   drrw   Add enumeration type reference for attributes
        03/06/12   drrw   Fix attributes for package parent element
        03/07/13   SL     Add highlighting for "Added/Changed" annotations. Add check mark for DBMapping. Add new node for DBMapping info. A Code re-factoring
        03/08/13   SL     Add setChoice info. Add attributes to root node.
        
    -->
    <xsl:param name="modelType">MindMap</xsl:param>
    <!-- values of MindMap, ArgoUML model, UML model -->
    <xsl:param name="details">none, content, cardinality</xsl:param>
    <!-- none, cardinality, content, description -->

    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
    <xsl:variable name="xslt-version">1.07</xsl:variable>
    <xsl:variable name="nlchar" select="'&#xA;'"/>
    <xsl:variable name="debug">0</xsl:variable>
    <!-- 0, 1, 2, 3, 4 or 5 -->
    <xsl:variable name="rootElement" select="as:CAM/as:AssemblyStructure/as:Structure/*[1]/name()"/>
    <xsl:variable name="EpochSeconds">
        <!-- xsl:sequence select="current-dateTime()"/ -->
        <xsl:sequence
            select="( current-dateTime() - xs:dateTime('1970-01-01T00:00:00') )
            div
            xs:dayTimeDuration('PT1S')
            * 1000
            "
        />
    </xsl:variable>

    <xsl:variable name="number_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="text_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="token_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="NMToken_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="ID_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="IDREF_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="date_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="datetime_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="time_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="duration_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="decimal_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="double_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="float_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="integer_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="byte_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="boolean_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="long_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="short_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="list_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="NCName_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="QName_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="any_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="uri_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="language_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="binary_uuid" select="uuid:randomUUID()"/>

    <xsl:variable name="ArgoUML-documentation">
        <UML:TaggedValue.type>
            <UML:TagDefinition
                href="http://argouml.org/profiles/uml14/default-uml14.xmi#.:000000000000087C"/>
        </UML:TaggedValue.type>
    </xsl:variable>
    <xsl:variable name="ArgoUML-usage">
        <UML:TaggedValue.type>
            <UML:TagDefinition
                href="http://argouml.org/profiles/uml14/default-uml14.xmi#-64--88-0-101--2259be85:11dd526880c:-8000:000000000000E4B3"
            />
        </UML:TaggedValue.type>
    </xsl:variable>
    <xsl:variable name="ArgoUML-semantics">
        <UML:TaggedValue.type>
            <UML:TagDefinition
                href="http://argouml.org/profiles/uml14/default-uml14.xmi#-64--88-0-101--2259be85:11dd526880c:-8000:000000000000E4B0"
            />
        </UML:TaggedValue.type>
    </xsl:variable>

    <xsl:variable name="rules">
        <xsl:for-each
            select="as:CAM/as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint">
            <xsl:choose>
                <xsl:when test="starts-with(@action,'makeRepeatable')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),')')"/>
                        <xsl:attribute name="type">repeatable</xsl:attribute>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="starts-with(@action,'makeOptional')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),')')"/>
                        <xsl:attribute name="type">optional</xsl:attribute>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="starts-with(@action,'restrictValues')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),',')"/>
                        <xsl:attribute name="type"
                            select="if(contains(@action,'''true''') and(contains(@action,'''false''') )) then 'boolean' else 'list'"/>
                        <xsl:attribute name="values"
                            select="replace(substring-before(substring-after(@action,','),')'),'''\|''','|')"/>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="starts-with(@action,'setDateMask')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),',')"/>
                        <xsl:attribute name="type" select="'datetype'"/>
                        <xsl:attribute name="mask"
                            select="substring-before(substring-after(@action,','),')')"/>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="starts-with(@action,'setNumberMask')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),',')"/>
                        <xsl:attribute name="type" select="'number'"/>
                        <xsl:attribute name="mask"
                            select="substring-before(substring-after(@action,','),')')"/>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="starts-with(@action,'setChoice')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),'/*')"/>
                        <xsl:attribute name="type">choice</xsl:attribute>
                        <xsl:attribute name="choices"
                            select="substring-before(substring-after(@action,'['),']')"/>
                    </xsl:element>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="descriptions">
        <xsl:for-each select="as:CAM/as:Extension/camed:annotation">
            <xsl:if test="camed:documentation/@type='Definition'">
                <xsl:element name="Note">
                    <xsl:attribute name="xpath" select="@item"/>
                    <xsl:value-of select="camed:documentation[@type='Definition']"/>
                </xsl:element>
            </xsl:if>
            <xsl:if test="camed:documentation/@type='KeyRef'">
                <xsl:element name="Key">
                    <xsl:attribute name="xpath" select="@item"/>
                    <xsl:value-of select="camed:documentation[@type='KeyRef']"/>
                </xsl:element>
            </xsl:if>
            <xsl:if test="camed:documentation/@type='Added' or camed:documentation/@type='Changed'">
                <xsl:element name="HasChanged">
                    <xsl:attribute name="xpath" select="@item"/>
                </xsl:element>
            </xsl:if>
            <xsl:if test="camed:documentation/@type='DBMapping'">
                <xsl:element name="HasMapping">
                    <xsl:attribute name="xpath" select="@item"/>
                    <xsl:value-of select="camed:documentation[@type='DBMapping']"/>
                </xsl:element>
            </xsl:if>
            
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="excludes">
        <xsl:for-each
            select="as:CAM/as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint">
            <xsl:choose>
                <xsl:when test="starts-with(@action,'exclude')">
                    <xsl:element name="Rule">
                        <xsl:attribute name="xpath"
                            select="substring-before(substring-after(@action,'('),')')"/>
                        <xsl:attribute name="type">exclude</xsl:attribute>
                    </xsl:element>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:for-each>
    </xsl:variable>

    <xsl:template match="/">
        <xsl:message>CAM model generator starting <xsl:value-of
                select="dateTime(current-date(),current-time())"/></xsl:message>
        <xsl:message>Version: <xsl:value-of select="$xslt-version"/></xsl:message>
        <xsl:message>Model mode: <xsl:value-of select="$modelType"/></xsl:message>
        <xsl:message>Content selections: <xsl:value-of select="$details"/></xsl:message>
        <xsl:message>From template structure: <xsl:value-of
                select="//as:AssemblyStructure/as:Structure/@ID"/></xsl:message>

        <xsl:choose>
            <xsl:when test="$modelType='MindMap'">
                <xsl:message>Writing model</xsl:message>
                <map version="0.9.0">
                    <xsl:comment>To view this file, download free mind mapping software FreeMind from http://freemind.sourceforge.net
                       Model autogenerated by CAM-2-model
                        Description: <xsl:value-of select="//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="//as:Header/as:Version"/> 
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <attribute_registry SHOW_ATTRIBUTES="hide"/>
                    <node BACKGROUND_COLOR="#ff6666" CREATED="{$EpochSeconds}"
                        ID="ID_{generate-id(node())}" MODIFIED="{$EpochSeconds}"
                        TEXT="{$rootElement}">
                        <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                        <xsl:for-each select="as:CAM/as:AssemblyStructure/as:Structure/*[1]/@*">
                            <xsl:message select="name()"/>
                            <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/@',name())"/>                    
                            <xsl:variable name="excluded"
                                select="as-xsl:excluded($xpath)"/>
                            <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}"
                                MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork"
                                TEXT="@{concat(name(.),$excluded)}">
                                <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded, '')"/>
                                <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                                <xsl:copy-of select="as-xsl:builtinAttr($xpath, $excluded)"/>
                                <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
                            </node>
                        </xsl:for-each>
                        <xsl:apply-templates select="//as:AssemblyStructure/as:Structure/*"
                            mode="MM"/>
                    </node>
                    <!-- xsl:copy-of select="$rules"/ -->
                    <!-- xsl:copy-of select="$descriptions"/ -->
                </map>
                <xsl:message>Model completed</xsl:message>
            </xsl:when>
            <xsl:when test="$modelType='UML model'">
                <xsl:message>Writing UML XMI model</xsl:message>
                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                <xsl:variable name="umlComment"
                    select="concat('Owner: ',//as:Header/as:Owner,'&#10; Created: ',//as:Header/as:DateTime,'&#10; Comment: ',//as:Header/as:Description,'&#10;')"/>
                <xsl:element name="xmi:XMI">
                    <xsl:namespace name="uml">http://schema.omg.org/spec/UML/2.1.2</xsl:namespace>
                    <xsl:namespace name="xmi">http://schema.omg.org/spec/XMI/2.1</xsl:namespace>
                    <xsl:attribute name="xmi:version">2.1</xsl:attribute>
                    <xsl:comment>To view this file, please use a modelling tool compatible with XMI v2.1 syntax.
                        Model autogenerated by CAM-2-model
                        Description: <xsl:value-of select="//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="//as:Header/as:Version"/> 
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <xmi:Documentation exporter="CAMeditor" exporterVersion="{$xslt-version}"/>
                    <uml:Model xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                        name="{//as:AssemblyStructure/as:Structure/name(child::node()[2])}"
                        visibility="public">
                        <ownedComment xmi:type="uml:Comment" xmi:id="{uuid:randomUUID()}"
                            body="{$umlComment}">
                            <annotatedElement xmi:idref="{$uuid}"/>
                        </ownedComment>
                        <xsl:apply-templates select="//as:AssemblyStructure/as:Structure" mode="XMI"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$text_uuid}"
                            xmi:uuid="{$text_uuid}" name="TextType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$number_uuid}"
                            xmi:uuid="{$number_uuid}" name="NumberType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$token_uuid}"
                            xmi:uuid="{$token_uuid}" name="TokenType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$NMToken_uuid}"
                            xmi:uuid="{$NMToken_uuid}" name="NMTokenType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$ID_uuid}"
                            xmi:uuid="{$ID_uuid}" name="IDType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$IDREF_uuid}"
                            xmi:uuid="{$IDREF_uuid}" name="IDRefType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$date_uuid}"
                            xmi:uuid="{$date_uuid}" name="DateType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$datetime_uuid}"
                            xmi:uuid="{$datetime_uuid}" name="DateTimeType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$time_uuid}"
                            xmi:uuid="{$time_uuid}" name="TimeType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$duration_uuid}"
                            xmi:uuid="{$duration_uuid}" name="DurationType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$decimal_uuid}"
                            xmi:uuid="{$decimal_uuid}" name="DecimalType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$double_uuid}"
                            xmi:uuid="{$double_uuid}" name="DoubleType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$float_uuid}"
                            xmi:uuid="{$float_uuid}" name="FloatType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$integer_uuid}"
                            xmi:uuid="{$integer_uuid}" name="IntegerType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$byte_uuid}"
                            xmi:uuid="{$byte_uuid}" name="ByteType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$boolean_uuid}"
                            xmi:uuid="{$boolean_uuid}" name="BooleanType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$long_uuid}"
                            xmi:uuid="{$long_uuid}" name="LongType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$short_uuid}"
                            xmi:uuid="{$short_uuid}" name="ShortType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$list_uuid}"
                            xmi:uuid="{$list_uuid}" name="ListType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$NCName_uuid}"
                            xmi:uuid="{$NCName_uuid}" name="NCNameType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$QName_uuid}"
                            xmi:uuid="{$QName_uuid}" name="QNameType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$any_uuid}"
                            xmi:uuid="{$any_uuid}" name="AnyType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$uri_uuid}"
                            xmi:uuid="{$uri_uuid}" name="URIType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$language_uuid}"
                            xmi:uuid="{$language_uuid}" name="LanguageType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$binary_uuid}"
                            xmi:uuid="{$binary_uuid}" name="BinaryType"/>
                    </uml:Model>
                </xsl:element>
                <xsl:message>Model completed</xsl:message>
            </xsl:when>
            <!-- xsl:when test="$modelType='Altova UML'">
                <xsl:message>Writing Altova UML model</xsl:message>
                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                <xsl:element name="xmi:XMI">
                    <xsl:namespace name="uml">http://schema.omg.org/spec/UML/2.1.2</xsl:namespace>
                    <xsl:namespace name="xmi">http://schema.omg.org/spec/XMI/2.1</xsl:namespace>
                    <xsl:attribute name="xmi:version">2.1</xsl:attribute>
                    <xsl:comment>To view this file, please use a modelling tool compatible with XMI v2.1 syntax.
                        Model autogenerated by CAM-2-model
                        Description: <xsl:value-of select="//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="//as:Header/as:Version"/> 
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <xmi:Documentation contact="www.sourceforge.net/projects/camprocessor" exporter="CAMeditor" exporterVersion="{$xslt-version}"/>
                    <uml:Package xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="Root">
                        <xsl:apply-templates select="//as:AssemblyStructure/as:Structure" mode="XMI"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$text_uuid}" xmi:uuid="{$text_uuid}" name="TextType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$number_uuid}" xmi:uuid="{$number_uuid}" name="NumberType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$token_uuid}" xmi:uuid="{$token_uuid}" name="TokenType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$NMToken_uuid}" xmi:uuid="{$NMToken_uuid}" name="NMTokenType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$ID_uuid}" xmi:uuid="{$ID_uuid}" name="IDType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$IDREF_uuid}" xmi:uuid="{$IDREF_uuid}" name="IDRefType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$date_uuid}" xmi:uuid="{$date_uuid}" name="DateType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$datetime_uuid}" xmi:uuid="{$datetime_uuid}" name="DateTimeType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$time_uuid}" xmi:uuid="{$time_uuid}" name="TimeType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$duration_uuid}" xmi:uuid="{$duration_uuid}" name="DurationType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$decimal_uuid}" xmi:uuid="{$decimal_uuid}" name="DecimalType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$double_uuid}" xmi:uuid="{$double_uuid}" name="DoubleType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$float_uuid}" xmi:uuid="{$float_uuid}" name="FloatType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$integer_uuid}" xmi:uuid="{$integer_uuid}" name="IntegerType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$byte_uuid}" xmi:uuid="{$byte_uuid}" name="ByteType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$boolean_uuid}" xmi:uuid="{$boolean_uuid}" name="BooleanType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$long_uuid}" xmi:uuid="{$long_uuid}" name="LongType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$short_uuid}" xmi:uuid="{$short_uuid}" name="ShortType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$list_uuid}" xmi:uuid="{$list_uuid}" name="ListType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$NCName_uuid}" xmi:uuid="{$NCName_uuid}" name="NCNameType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$QName_uuid}" xmi:uuid="{$QName_uuid}" name="QNameType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$any_uuid}" xmi:uuid="{$any_uuid}" name="AnyType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$uri_uuid}" xmi:uuid="{$uri_uuid}" name="URIType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$language_uuid}" xmi:uuid="{$language_uuid}" name="LanguageType"/>
                        <packagedElement xmi:type="uml:Class" xmi:id="{$binary_uuid}" xmi:uuid="{$binary_uuid}" name="BinaryType"/>
                    </uml:Package>
                </xsl:element>
                <xsl:message>Model completed</xsl:message>
            </xsl:when -->
            <xsl:when test="$modelType='ArgoUML model'">
                <xsl:message>Writing ArgoUML model</xsl:message>
                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                <XMI xmi.version="1.2" xmlns:UML="org.omg.xmi.namespace.UML"
                    timestamp="{dateTime(current-date(),current-time())}">
                    <xsl:comment>To view this file, please use ArgoUML modelling tool.
                        Model autogenerated by CAM-2-model
                        Description: <xsl:value-of select="//as:Header/as:Description"/> 
                        Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                        Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                        Version: <xsl:value-of select="//as:Header/as:Version"/> 
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <XMI.header>
                        <XMI.documentation>
                            <XMI.exporter>ArgoUML from CAMeditor tool</XMI.exporter>
                            <XMI.exporterVersion><xsl:value-of select="$xslt-version"/> xslt
                                exporter CAM-2-model $</XMI.exporterVersion>
                        </XMI.documentation>
                        <XMI.metamodel xmi.name="UML" xmi.version="1.4"/>
                    </XMI.header>
                    <XMI.content>
                        <UML:Model xmi.id="{$uuid}" name="{//as:Header/as:Description}"
                            isSpecification="false" isRoot="false" isLeaf="false" isAbstract="false">
                            <xsl:apply-templates select="//as:AssemblyStructure/as:Structure"
                                mode="ArgoUML"/>
                        </UML:Model>
                    </XMI.content>
                </XMI>
                <xsl:message>Model completed</xsl:message>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">ERROR: Unknown model output type: <xsl:value-of
                        select="$modelType"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

    <xsl:template match="*" mode="MM">
        <!-- MM branch level nodes -->
        <xsl:for-each select="*">
            <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>            
            <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/>
            <xsl:variable name="choice" select="as-xsl:choice(name(), concat('/',name(parent::node())))"/>
            <xsl:variable name="content">
                <xsl:copy-of select="as-xsl:getContent($xpath)"/>
            </xsl:variable>
            <xsl:variable name="description" select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
            <xsl:variable name="excluded"
                select="as-xsl:excluded($xpath)"/>
            <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
                POSITION="right" STYLE="{if(count(child::*) = 0) then 'fork' else 'bubble'}"
                TEXT="{concat($choice, name(.),' ',$cardinality,$excluded)}">
                <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded, $choice)"/>
                <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                <xsl:copy-of select="as-xsl:builtinAttr($xpath, $excluded)"/>
                <xsl:if test="$content/attribute and(contains($details,'content'))">
                    <xsl:copy-of select="$content"/>
                </xsl:if>
                <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))">
                    <richcontent TYPE="NOTE">
                        <html>
                            <head/>
                            <body>
                                <p>
                                    <xsl:value-of select="$description[1]"/>
                                </p>
                            </body>
                        </html>
                    </richcontent>
                </xsl:if>
                <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
                <xsl:for-each select="@*">
                    <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/@',name())"/>                    
                    <xsl:variable name="excluded"
                        select="as-xsl:excluded($xpath)"/>
                    <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}"
                        MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork"
                        TEXT="@{concat(name(.),$excluded)}">
                        <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded, '')"/>
                        <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                        <xsl:copy-of select="as-xsl:builtinAttr($xpath, $excluded)"/>
                        <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
                    </node>
                </xsl:for-each>
                <xsl:if test="$excluded = ''">
                    <xsl:apply-templates select="*" mode="MMchild"/>
                </xsl:if>
            </node>
            <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="*" mode="MMchild">
        <!-- leaf nodes -->
        <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
        <xsl:variable name="description" select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>       
        <xsl:variable name="choice" select="as-xsl:choice(name(), concat('/', name(../..),'/',name(parent::node())))"/>
        <xsl:variable name="excluded"
            select="as-xsl:excluded($xpath)"/>
        <xsl:variable name="content">
            <xsl:copy-of select="as-xsl:getContent($xpath)"/>
        </xsl:variable>
        <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 
        <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
            POSITION="right" STYLE="{if(count(child::*) = 0) then 'fork' else 'bubble'}"
            TEXT="{concat($choice, name(.),' ',$cardinality,$excluded)}">
            <xsl:if test="child::*">
                <xsl:attribute name="FOLDED">true</xsl:attribute>
            </xsl:if>
            <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded, $choice)"/>
            <xsl:choose>
                <xsl:when test="count($descriptions/HasChanged[ends-with(@xpath,$xpath)]) &gt; 0">
                    <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                </xsl:when>
                <xsl:otherwise>
                    <font BOLD="{if(count(child::*) = 0) then 'false' else 'true'}" NAME="SansSerif"
                        SIZE="12"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:copy-of select="as-xsl:builtinAttr($xpath, $excluded)"/>
            <xsl:if test="$content/attribute and(contains($details,'content'))">
                <xsl:copy-of select="$content"/>
            </xsl:if>
            <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))">
                <richcontent TYPE="NOTE">
                    <html>
                        <head/>
                        <body>
                            <p>
                                <xsl:value-of select="$description[1]"/>
                            </p>
                        </body>
                    </html>
                </richcontent>
            </xsl:if>
            <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
            <xsl:for-each select="@*">
                <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/@',name())"/>
                <xsl:variable name="excluded"
                    select="as-xsl:excluded($xpath)"/>
                <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
                    POSITION="right" STYLE="fork" TEXT="@{concat(name(.),$excluded)}">
                    <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded,'')"/>
                    <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                    <xsl:choose>
                        <xsl:when test="count($descriptions/HasMapping[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="button_ok"/></xsl:when>
                    </xsl:choose>
                    <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
                </node>
            </xsl:for-each>
            <xsl:if test="$excluded=''">
                <xsl:for-each select="*">
                    <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
                    <xsl:variable name="description"
                        select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
                    <xsl:variable name="excluded"
                        select="as-xsl:excluded($xpath)"/>
                    <xsl:variable name="content">
                        <xsl:copy-of select="as-xsl:getContent($xpath)"/>
                    </xsl:variable>
                    <xsl:variable name="choice" select="as-xsl:choice(name(), concat('/', name(../..),'/',name(parent::node())))"/>
                    <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 
                    <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}"
                        MODIFIED="{$EpochSeconds}" POSITION="right"
                        STYLE="{if(count(child::*) = 0) then 'fork' else 'bubble'}"
                        TEXT="{concat($choice, name(.),' ',$cardinality,$excluded)}">
                        <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded,$choice)"/>
                        <xsl:choose>
                            <xsl:when test="count($descriptions/HasChanged[ends-with(@xpath,$xpath)]) &gt; 0">
                                <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <font BOLD="{if(count(child::*) = 0) then 'false' else 'true'}" NAME="SansSerif"
                                    SIZE="12"/>
                            </xsl:otherwise>
                        </xsl:choose>
                        <xsl:copy-of select="as-xsl:builtinAttr($xpath, $excluded)"/>
                        <xsl:if test="$content/attribute and(contains($details,'content'))">
                            <xsl:copy-of select="$content"/>
                        </xsl:if>
                        <xsl:if
                            test="$description[1] &gt; ' ' and(contains($details,'description'))">
                            <richcontent TYPE="NOTE">
                                <html>
                                    <head/>
                                    <body>
                                        <p>
                                            <xsl:value-of select="$description[1]"/>
                                        </p>
                                    </body>
                                </html>
                            </richcontent>
                        </xsl:if>
                        <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
                        <xsl:for-each select="@*">
                            <xsl:variable name="xpath"
                                select="concat('/',name(parent::node()),'/@',name())"/>
                            <xsl:variable name="excluded"
                                select="as-xsl:excluded($xpath)"/>                      
                            <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}"
                                MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork"
                                TEXT="@{concat(name(.),$excluded)}">
                                <xsl:copy-of select="as-xsl:colorAttr($xpath, $excluded,'')"/>
                                <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                                <xsl:copy-of select="as-xsl:builtinAttr($xpath, $excluded)"/>
                                <xsl:copy-of select="as-xsl:dbMapping(., $descriptions/HasMapping[ends-with(@xpath,$xpath)])"/>
                            </node>
                        </xsl:for-each>
                        <xsl:if test="$excluded=''">
                            <xsl:apply-templates select="*" mode="MMchild"/>
                        </xsl:if>
                    </node>
                    <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->
                </xsl:for-each>
            </xsl:if>
        </node>
    </xsl:template>

    <xsl:template match="*" mode="XMI">
        <!-- XMI branch level nodes -->
        <xsl:for-each select="*">
            <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
           <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 
            <xsl:variable name="content">
                <xsl:copy-of select="as-xsl:getContent($xpath)"/>
            </xsl:variable>
            <xsl:variable name="description" select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
            <xsl:variable name="excluded"
                select="as-xsl:excluded($xpath)"/>
            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
            <packagedElement xmi:type="uml:Package" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                name="{name(.)}">
                <xsl:if test="not($cardinality = '')">
                    <uml:Multiplicity>
                        <uml:Multiplicity.range>
                            <uml:MultiplicityRange
                                lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                                upper="{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                        </uml:Multiplicity.range>
                    </uml:Multiplicity>
                </xsl:if>

                <!-- output any details -->
                <xsl:call-template name="outputXMIdetails">
                    <xsl:with-param name="xpath" select="$xpath"/>
                    <xsl:with-param name="version" select="'2.1'"/>
                    <xsl:with-param name="parent-uuid" select="$uuid"/>
                    <xsl:with-param name="attr"/>
                </xsl:call-template>

                <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                    <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                    <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                    <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if>
                    <xsl:if test="$content/attribute and(contains($details,'content'))"><xsl:copy-of select="$content"/></xsl:if>
                    <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))"><richcontent TYPE="NOTE"><html><head/><body><p><xsl:value-of select="$description[1]"/></p></body></html></richcontent></xsl:if -->
          
                 <xsl:if test="count(@*) &gt; 0">
                     <xsl:variable name="abstract-uuid" select="uuid:randomUUID()"/>
                     <packagedElement xmi:type="uml:Class" xmi:id="{$abstract-uuid}" name="{name(.)}_attributes" visibility="public" association="{$uuid}">
                    <xsl:for-each select="@*"> 
                        <xsl:variable name="xpath"
                            select="concat('/',name(parent::node()),'/@',name())"/>
                        <xsl:variable name="excluded"
                            select="as-xsl:excluded($xpath)"/>
                        <xsl:variable name="this-uuid" select="uuid:randomUUID()"/>
                        <xsl:choose>
                            <xsl:when test="contains($details,'content') and($rules/Rule[@values][ends-with(@xpath,$xpath)])"><!-- optionally declare enumeration for attribute -->
                                <xsl:variable name="enum-uuid" select="uuid:randomUUID()"/>
                                <xsl:variable name="ruleValues"
                                    select="$rules/Rule[@values][ends-with(@xpath,$xpath)]/@values"/>
                                <nestedClassifier xmi:type='uml:Class' xmi:id='{$enum-uuid}' name='{name(.)}_enumerations' visibility='public'>
                                    <generalization xmi:type='uml:Generalization' xmi:id='{uuid:randomUUID()}' general="{$text_uuid}">
                                        <!-- general xmi:type='uml:Class' type='{$text_uuid}'/ -->
                                    </generalization>
                                    <!-- xsl:comment> <xsl:value-of select="$ruleValues[1]"/> - <xsl:value-of select="tokenize($ruleValues[1],'\|')"/></xsl:comment -->
                                    <xsl:for-each select="tokenize($ruleValues[1],'\|')">
                                        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                                        <ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" name="{replace(.,'''','')}" visibility="private" aggregation="shared"/>
                                    </xsl:for-each>
                                </nestedClassifier>
                                <ownedAttribute xmi:type="uml:Property" xmi:id="{$this-uuid}"
                                    xmi:uuid="{$this-uuid}" name="{name(.)}" visibility="public"
                                    type="{$enum-uuid}">
                                    <xsl:call-template name="outputXMIdetails">
                                        <xsl:with-param name="xpath" select="$xpath"/>
                                        <xsl:with-param name="version" select="'2.1'"/>
                                        <xsl:with-param name="parent-uuid" select="$uuid"/>
                                        <xsl:with-param name="attr">true</xsl:with-param>
                                    </xsl:call-template>
                                </ownedAttribute>
                            </xsl:when>
                            <xsl:otherwise>
                                <ownedAttribute xmi:type="uml:Property" xmi:id="{$this-uuid}"
                                    xmi:uuid="{$this-uuid}" name="{name(.)}" visibility="public"
                                    type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                                    <xsl:call-template name="outputXMIdetails">
                                        <xsl:with-param name="xpath" select="$xpath"/>
                                        <xsl:with-param name="version" select="'2.1'"/>
                                        <xsl:with-param name="parent-uuid" select="$uuid"/>
                                        <xsl:with-param name="attr">true</xsl:with-param>
                                    </xsl:call-template>
                                </ownedAttribute>
                            </xsl:otherwise>
                        </xsl:choose>

                        <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                        <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                        <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                        <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if -->
                    </xsl:for-each>
                     </packagedElement>
                 </xsl:if>

                <xsl:if test="$excluded = ''">
                    <xsl:apply-templates select="*" mode="XMIchild"/>
                </xsl:if>

            </packagedElement>
            <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->
        </xsl:for-each>
    </xsl:template>


    <xsl:template match="*" mode="XMIchild">
        <!-- leaf nodes -->
        <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
        <xsl:variable name="description" select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
        <xsl:variable name="excluded"
            select="as-xsl:excluded($xpath)"/>
        <xsl:variable name="content">
            <xsl:copy-of select="as-xsl:getContent($xpath)"/>
        </xsl:variable>
        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
        <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 

        <packagedElement xmi:type="{if(count(child::*) = 0) then 'uml:Class' else 'uml:Package'}"
            xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{name(.)}">
            <xsl:if test="not($cardinality = '')">
                <uml:Multiplicity>
                    <uml:Multiplicity.range>
                        <uml:MultiplicityRange
                            lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                            upper="{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                    </uml:Multiplicity.range>
                </uml:Multiplicity>
            </xsl:if>

            <xsl:copy-of select="as-xsl:content($content)"/>
            <!-- output data type model -->
            <!-- xsl:if test="child::*"><xsl:attribute name="FOLDED">true</xsl:attribute></xsl:if>
            <xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
            <font BOLD="{if(count(child::*) = 0) then 'false' else 'true'}" NAME="SansSerif" SIZE="12"/>
            <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
            <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if>
            <xsl:if test="$content/attribute and(contains($details,'content'))"><xsl:copy-of select="$content"/></xsl:if>
            <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))"><richcontent TYPE="NOTE"><html><head/><body><p><xsl:value-of select="$description[1]"/></p></body></html></richcontent></xsl:if -->
            <!-- xsl:if
                test="count(child::*) &gt; 0 or(count(@*) &gt; 0)">
                
                <xsl:variable name="this-uuid" select="uuid:randomUUID()"/>
                <xsl:message> - doc 2 <xsl:value-of select="name(.)"/></xsl:message>
                <ownedComment xmi:type="uml:Comment" xmi:id="{$this-uuid}" xmi:uuid="{$this-uuid}"
                    body="XML attributes shown as documentation for package {name(.)}">
                    <annotatedElement xmi:idref="{$uuid}"/>
                </ownedComment>
            </xsl:if -->

            <xsl:if test="count(@*) &gt; 0">
                <xsl:choose>
                    <xsl:when test="count(child::*) = 0">
                        <xsl:for-each select="@*">
                            <xsl:variable name="xpath"
                                select="concat('/',name(parent::node()),'/@',name())"/>
                            <xsl:variable name="excluded"
                                select="as-xsl:excluded($xpath)"/>
                            <xsl:variable name="this-uuid" select="uuid:randomUUID()"/>

                            <xsl:choose>
                                <xsl:when test="contains($details,'content') and($rules/Rule[@values][ends-with(@xpath,$xpath)])"><!-- optionally declare enumeration for attribute -->
                                    <xsl:variable name="enum-uuid" select="uuid:randomUUID()"/>
                                    <xsl:variable name="ruleValues"
                                        select="$rules/Rule[@values][ends-with(@xpath,$xpath)]/@values"/>
                                    <nestedClassifier xmi:type='uml:Class' xmi:id='{$enum-uuid}' name='{name(.)}_enumerations' visibility='public'>
                                        <generalization xmi:type='uml:Generalization' xmi:id='{uuid:randomUUID()}' general='{$text_uuid}'>
                                            <!-- general xmi:type='uml:Class' type='{$text_uuid}'/ -->
                                        </generalization>
                                        <!-- xsl:comment> <xsl:value-of select="$ruleValues[1]"/> - <xsl:value-of select="tokenize($ruleValues[1],'\|')"/></xsl:comment -->
                                        <xsl:for-each select="tokenize($ruleValues[1],'\|')">
                                            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                                            <ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" name="{replace(.,'''','')}" visibility="private" aggregation="shared"/>
                                        </xsl:for-each>
                                    </nestedClassifier>
                                    <ownedAttribute xmi:type="uml:Property" xmi:id="{$this-uuid}"
                                        xmi:uuid="{$this-uuid}" name="{name(.)}" visibility="public"
                                        type="{$enum-uuid}">
                                        <xsl:call-template name="outputXMIdetails">
                                            <xsl:with-param name="xpath" select="$xpath"/>
                                            <xsl:with-param name="version" select="'2.1'"/>
                                            <xsl:with-param name="parent-uuid" select="$uuid"/>
                                            <xsl:with-param name="attr">true</xsl:with-param>
                                        </xsl:call-template>
                                    </ownedAttribute>
                                </xsl:when>
                                <xsl:otherwise>
                                    <ownedAttribute xmi:type="uml:Property" xmi:id="{$this-uuid}"
                                        xmi:uuid="{$this-uuid}" name="{name(.)}" visibility="public"
                                        type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                                        <xsl:call-template name="outputXMIdetails">
                                            <xsl:with-param name="xpath" select="$xpath"/>
                                            <xsl:with-param name="version" select="'2.1'"/>
                                            <xsl:with-param name="parent-uuid" select="$uuid"/>
                                            <xsl:with-param name="attr">true</xsl:with-param>
                                        </xsl:call-template>
                                    </ownedAttribute>
                                </xsl:otherwise>
                            </xsl:choose>
                            <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                    <font BOLD="false" NAME="SansSerif" SIZE="10"/ -->
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- Now create a container class to hold the attributes -->
                        <xsl:comment>Container class for attributes if not package output</xsl:comment>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>

            <xsl:if test="$excluded=''">
                <xsl:for-each select="*">
                    <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
                    <xsl:variable name="description"
                        select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
                    <xsl:variable name="excluded"
                        select="as-xsl:excluded($xpath)"/>
                    <xsl:variable name="content">
                        <xsl:copy-of select="as-xsl:getContent($xpath)"/>
                    </xsl:variable>
                    <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 
                    <xsl:choose>
                        <xsl:when test="count(child::*) = 0">
                            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                            <packagedElement
                                xmi:type="{if(count(child::*) = 0) then 'uml:Class' else 'uml:Package'}"
                                xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{name(.)}">
                                <xsl:if test="not($cardinality = '')">
                                    <uml:Multiplicity>
                                        <uml:Multiplicity.range>
                                            <uml:MultiplicityRange
                                                lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                                                upper="{if(contains($cardinality,'*')) then '-1' else '1'}"
                                            />
                                        </uml:Multiplicity.range>
                                    </uml:Multiplicity>
                                </xsl:if>
                                <xsl:copy-of select="as-xsl:content($content)"/>
                                <!-- output data type model -->
                            </packagedElement>
                            <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                            <font BOLD="{if(count(child::*) = 0) then 'false' else 'true'}" NAME="SansSerif" SIZE="12"/>
                            <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                            <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if>
                            <xsl:if test="$content/attribute and(contains($details,'content'))"><xsl:copy-of select="$content"/></xsl:if>
                            <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))"><richcontent TYPE="NOTE"><html><head/><body><p><xsl:value-of select="$description[1]"/></p></body></html></richcontent></xsl:if -->
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                            <packagedElement
                                xmi:type="{if(count(child::*) = 0) then 'uml:Class' else 'uml:Package'}"
                                xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{name(.)}">
                                <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                                <font BOLD="{if(count(child::*) = 0) then 'false' else 'true'}" NAME="SansSerif" SIZE="12"/>
                                <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                                <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if>
                                <xsl:if test="$content/attribute and(contains($details,'content'))"><xsl:copy-of select="$content"/></xsl:if>
                                <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))"><richcontent TYPE="NOTE"><html><head/><body><p><xsl:value-of select="$description[1]"/></p></body></html></richcontent></xsl:if -->
                                <xsl:if test="not($cardinality = '')">
                                    <uml:Multiplicity>
                                        <uml:Multiplicity.range>
                                            <uml:MultiplicityRange
                                                lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                                                upper="{if(contains($cardinality,'*')) then '-1' else '1'}"
                                            />
                                        </uml:Multiplicity.range>
                                    </uml:Multiplicity>
                                </xsl:if>
                                <xsl:copy-of select="as-xsl:content($content)"/>
                                <!-- output data type model -->

                                <xsl:if test="count(@*) &gt; 0">
                                    <!-- Now create a container class to hold the attributes -->
                                    <xsl:choose>
                                        <xsl:when test="count(child::*) = 0">
                                            <xsl:for-each select="@*">
                                                <xsl:variable name="xpath"
                                                  select="concat('/',name(parent::node()),'/@',name())"/>
                                                <xsl:variable name="excluded"
                                                  select="as-xsl:excluded($xpath)"/>
                                                <xsl:variable name="this-uuid" select="uuid:randomUUID()"/>

                                                <xsl:choose>
                                                    <xsl:when test="contains($details,'content') and($rules/Rule[@values][ends-with(@xpath,$xpath)])"><!-- optionally declare enumeration for attribute -->
                                                        <xsl:variable name="enum-uuid" select="uuid:randomUUID()"/>
                                                        <xsl:variable name="ruleValues"
                                                            select="$rules/Rule[@values][ends-with(@xpath,$xpath)]/@values"/>
                                                        <nestedClassifier xmi:type='uml:Class' xmi:id='{$enum-uuid}' name='{name(.)}_enumerations' visibility='public'>
                                                            <generalization xmi:type='uml:Generalization' xmi:id='{uuid:randomUUID()}' general='{$text_uuid}'>
                                                                <!-- general xmi:type='uml:Class' type='{$text_uuid}'/ -->
                                                            </generalization>
                                                            <!-- xsl:comment> <xsl:value-of select="$ruleValues[1]"/> - <xsl:value-of select="tokenize($ruleValues[1],'\|')"/></xsl:comment -->
                                                            <xsl:for-each select="tokenize($ruleValues[1],'\|')">
                                                                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                                                                <ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" name="{replace(.,'''','')}" visibility="private" aggregation="shared"/>
                                                            </xsl:for-each>
                                                        </nestedClassifier>
                                                        <ownedAttribute xmi:type="uml:Property" xmi:id="{$this-uuid}"
                                                            xmi:uuid="{$this-uuid}" name="{name(.)}" visibility="public"
                                                            type="{$enum-uuid}">
                                                            <xsl:call-template name="outputXMIdetails">
                                                                <xsl:with-param name="xpath" select="$xpath"/>
                                                                <xsl:with-param name="version" select="'2.1'"/>
                                                                <xsl:with-param name="parent-uuid" select="$uuid"/>
                                                                <xsl:with-param name="attr">true</xsl:with-param>
                                                            </xsl:call-template>
                                                        </ownedAttribute>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                        <ownedAttribute xmi:type="uml:Property" xmi:id="{$this-uuid}"
                                                            xmi:uuid="{$this-uuid}" name="{name(.)}" visibility="public"
                                                            type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                                                            <xsl:call-template name="outputXMIdetails">
                                                                <xsl:with-param name="xpath" select="$xpath"/>
                                                                <xsl:with-param name="version" select="'2.1'"/>
                                                                <xsl:with-param name="parent-uuid" select="$uuid"/>
                                                                <xsl:with-param name="attr">true</xsl:with-param>
                                                            </xsl:call-template>
                                                        </ownedAttribute>
                                                    </xsl:otherwise>
                                                </xsl:choose>

                                                <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                                        <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                                        <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                                        <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if -->
                                            </xsl:for-each>
                                        </xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:if>

                                <xsl:if test="$excluded=''">
                                    <xsl:apply-templates select="*" mode="XMIchild"/>
                                </xsl:if>
                            </packagedElement>
                        </xsl:otherwise>
                    </xsl:choose>

                    <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->
                </xsl:for-each>
            </xsl:if>

        </packagedElement>
    </xsl:template>

    <xsl:template name="outputXMIdetails">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <xsl:param name="attr"/>

        <xsl:call-template name="outputXMIcomment">
            <xsl:with-param name="xpath" select="$xpath"/>
            <xsl:with-param name="version" select="'2.1'"/>
            <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
        </xsl:call-template>

        <xsl:call-template name="outputXMIrule">
            <xsl:with-param name="xpath" select="$xpath"/>
            <xsl:with-param name="version" select="'2.1'"/>
            <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
        </xsl:call-template>

        <xsl:if test="not($attr='true')">
            <!-- Don't output for attributes -->
            <xsl:call-template name="outputXMItype">
                <xsl:with-param name="xpath" select="$xpath"/>
                <xsl:with-param name="version" select="'2.1'"/>
                <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
            </xsl:call-template>
        </xsl:if>

        <xsl:if test="not($attr='true')"><!-- Don't output for attributes, it has to be as a named type instead -->
            <xsl:call-template name="outputXMIenumeration">
                <xsl:with-param name="xpath" select="$xpath"/>
                <xsl:with-param name="version" select="'2.1'"/>
                <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
            </xsl:call-template>
        </xsl:if>

    </xsl:template>

    <xsl:template name="outputXMIcomment">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'description') and($descriptions/Note[ends-with(@xpath,$xpath)])">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <xsl:variable name="description"
                        select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
                    <ownedComment xmi:type="uml:Comment" xmi:id="{$uuid}" xmi:uuid="{$uuid}">
                        <body>
                            <xsl:value-of select="$description"/>
                        </body>
                        <annotatedElement xmi:idref="{$parent-uuid}"/>
                    </ownedComment>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputXMIrule">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'cardinality') and($rules/Rule[@type='optional'][ends-with(@xpath,$xpath)] or($rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]))">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <uml:Multiplicity>
                        <uml:MultiplicityRange
                            lower="{if($rules/Rule[@type='optional'][ends-with(@xpath,$xpath)]) then '0' else '1'}"
                            upper="{if($rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]) then '*' else '1'}"
                        />
                    </uml:Multiplicity>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputXMItype">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'content') and($rules/Rule[not(@type='optional')][ends-with(@xpath,$xpath)])">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <xsl:variable name="typeOf"
                        select="$rules/Rule[not(@type='optional')][ends-with(@xpath,$xpath)]/@type"/>
                    <nestedClassifier xmi:type="uml:DataType" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                        name="{$typeOf[1]}"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputXMIenumeration">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'content') and($rules/Rule[@values][ends-with(@xpath,$xpath)])">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <xsl:variable name="ruleValues"
                        select="$rules/Rule[@values][ends-with(@xpath,$xpath)]/@values"/>
                    <nestedClassifier xmi:type="uml:Enumeration" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                        name="list">
                        <!-- xsl:comment> <xsl:value-of select="$ruleValues[1]"/> - <xsl:value-of select="tokenize($ruleValues[1],'\|')"/></xsl:comment -->
                        <xsl:for-each select="tokenize($ruleValues[1],'\|')">
                            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                            <ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" name="{replace(.,'''','')}" visibility="private" aggregation="shared"/>
                            <!-- ownedLiteral xmi:type="uml:EnumerationLiteral" xmi:id="{$uuid}"
                                xmi:uuid="{$uuid}" name="{replace(.,'''','')}"/ -->
                        </xsl:for-each>
                    </nestedClassifier>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template match="*" mode="ArgoUML">
        <!-- Argo XMI branch level nodes -->
        <xsl:for-each select="*">
            <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
           <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 
            <!-- xsl:variable name="content">
                <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
                    <xsl:variable name="type" select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
                    <xsl:variable name="mask" select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"></xsl:variable>
                    <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
                </xsl:if>
            </xsl:variable -->
            <xsl:variable name="description" select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
            <xsl:variable name="excluded"
                select="as-xsl:excluded($xpath)"/>
            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
            <UML:Namespace.ownedElement>
                <UML:Class xmi.id="{$uuid}" name="{name(.)}" visibility="public"
                    isSpecification="false" isRoot="true" isLeaf="false" isAbstract="false"
                    isActive="false">
                    <xsl:if
                        test="contains($details,'description') and($descriptions/Note[@xpath = concat('/',$xpath)][1])">
                        <xsl:variable name="uuid-desc" select="uuid:randomUUID()"/>
                        <UML:ModelElement.taggedValue>
                            <UML:TaggedValue xmi.id="{$uuid-desc}" isSpecification="false">
                                <UML:TaggedValue.dataValue>
                                    <xsl:value-of
                                        select="$descriptions/Note[@xpath = concat('/',$xpath)][1]"
                                    />
                                </UML:TaggedValue.dataValue>
                                <xsl:copy-of select="$ArgoUML-documentation"/>
                            </UML:TaggedValue>
                        </UML:ModelElement.taggedValue>
                    </xsl:if>

                    <xsl:if test="not($cardinality = '')">
                        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                        <UML:StructuralFeature.multiplicity>
                            <UML:Multiplicity xmi.id="{$uuid}">
                                <UML:Multiplicity.range>
                                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                                    <UML:MultiplicityRange xmi.id="{$uuid}"
                                        lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                                        upper="{if(contains($cardinality,'*')) then '-1' else '1'}"
                                    />
                                </UML:Multiplicity.range>
                            </UML:Multiplicity>
                        </UML:StructuralFeature.multiplicity>
                    </xsl:if>

                    <!-- output any details -->
                    <xsl:call-template name="outputArgoUMLdetails">
                        <xsl:with-param name="xpath" select="$xpath"/>
                        <xsl:with-param name="version" select="'2.1'"/>
                        <xsl:with-param name="parent-uuid" select="$uuid"/>
                    </xsl:call-template>

                    <!-- xsl:for-each select="@*">
                    <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/@',name())"/>
                    <xsl:variable name="excluded" select="as-xsl:excluded($xpath)" />
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/ -->

                    <!-- output any annotations -->
                    <!-- ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{name(.)}" visibility="public" type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                        <xsl:call-template name="outputArgoUMLdetails">
                            <xsl:with-param name="xpath" select="$xpath"/>
                            <xsl:with-param name="version" select="'2.1'"/>
                            <xsl:with-param name="parent-uuid" select="$uuid"/>
                        </xsl:call-template>
                    </ownedAttribute -->
                    <!-- /xsl:for-each -->
                    <xsl:if test="$excluded = ''">
                        <xsl:apply-templates select="*" mode="ArgoUMLchild"/>
                    </xsl:if>

                </UML:Class>
            </UML:Namespace.ownedElement>
            <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->
        </xsl:for-each>
        <UML:Namespace.ownedElement>
            <UML:Class xmi.id="{$text_uuid}" name="TextType" visibility="public"
                isSpecification="false" isRoot="false" isLeaf="false" isAbstract="false"
                isActive="false"> </UML:Class>
        </UML:Namespace.ownedElement>
    </xsl:template>

    <xsl:template match="*" mode="ArgoUMLchild">
        <!-- leaf nodes -->
        <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/',name())"/>
        <xsl:variable name="description" select="$descriptions/Note[ends-with(@xpath,$xpath)]"/>
        <xsl:variable name="excluded"
            select="as-xsl:excluded($xpath)"/>
        <!-- xsl:variable name="content">
            <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
                <xsl:variable name="type" select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
                <xsl:variable name="mask" select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"></xsl:variable>
                <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
            </xsl:if>
        </xsl:variable -->
        <xsl:variable name="class_uuid" select="uuid:randomUUID()"/>
        <xsl:variable name="cardinality" select="as-xsl:cardinality($xpath)"/> 

        <UML:Namespace.ownedElement>
            <UML:Class xmi.id="{$class_uuid}" name="{name(.)}" visibility="public"
                isSpecification="false" isRoot="false" isLeaf="false" isAbstract="false"
                isActive="false">
                <xsl:if
                    test="contains($details,'description') and($descriptions/Note[ends-with(@xpath,$xpath)][1])">
                    <xsl:variable name="uuid-desc" select="uuid:randomUUID()"/>
                    <UML:ModelElement.taggedValue>
                        <UML:TaggedValue xmi.id="{$uuid-desc}" isSpecification="false">
                            <UML:TaggedValue.dataValue>
                                <xsl:value-of
                                    select="$descriptions/Note[ends-with(@xpath,$xpath)][1]"/>
                            </UML:TaggedValue.dataValue>
                            <xsl:copy-of select="$ArgoUML-documentation"/>
                        </UML:TaggedValue>
                    </UML:ModelElement.taggedValue>
                </xsl:if>
                <!-- UML:Attribute xmi.id = "{$uuid}"
            name = "{name(.)}" visibility = "public" isSpecification = "false"
            ownerScope = "instance" changeability = "changeable" targetScope = "instance" -->
                <xsl:if test="not($cardinality = '')">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <!-- UML:StructuralFeature.multiplicity>
                    <UML:Multiplicity xmi.id = "{$uuid}">
                        <UML:Multiplicity.range>
                            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                            <UML:MultiplicityRange xmi.id = "{$uuid}"
                                lower = "{if(contains($cardinality,'?')) then '0' else '1'}" upper = "{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                        </UML:Multiplicity.range>
                    </UML:Multiplicity>
                    </UML:StructuralFeature.multiplicity -->
                    <UML:ModelElement.taggedValue>
                        <UML:TaggedValue xmi.id="{$uuid}" isSpecification="false">
                            <UML:TaggedValue.dataValue>Cardinality: lower=<xsl:value-of
                                    select="if(contains($cardinality,'?')) then '0' else '1'"/>
                                    upper=<xsl:value-of
                                    select="if(contains($cardinality,'*')) then '-1' else '1'"
                                /></UML:TaggedValue.dataValue>
                            <xsl:copy-of select="$ArgoUML-usage"/>
                        </UML:TaggedValue>
                    </UML:ModelElement.taggedValue>
                </xsl:if>
                <UML:StructuralFeature.type>
                    <UML:Class xmi.idref="{$text_uuid}"/>
                </UML:StructuralFeature.type>
                <!-- Now output child structure components  -->
                <!-- UML:Classifier.feature -->
                <xsl:apply-templates select="*" mode="ArgoUMLchild"/>
                <!-- /UML:Classifier.feature -->
                <!-- /UML:Attribute -->
            </UML:Class>
        </UML:Namespace.ownedElement>
    </xsl:template>

    <xsl:template name="outputArgoUMLdetails">
        <xsl:param name="parent-uuid"/>
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <!-- empty for now -->
    </xsl:template>

    <xsl:function name="as-xsl:uuid-type">
        <xsl:param name="rules"/>
        <xsl:param name="nodeType"/>

        <xsl:choose>
            <xsl:when test="$rules[@type = 'list']">
                <xsl:value-of select="$list_uuid"/>
            </xsl:when>
            <xsl:when test="$rules[@type = 'number']">
                <xsl:value-of select="$decimal_uuid"/>
            </xsl:when>
            <xsl:when test="$rules[@type = 'datetype']">
                <xsl:value-of select="$datetime_uuid"/>
            </xsl:when>
            <xsl:when test="$nodeType = 'child'">
                <xsl:value-of select="$text_uuid"/>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>

    </xsl:function>

    <xsl:function name="as-xsl:uuid-dicttype">
        <xsl:param name="data-type"/>
        <xsl:choose>
            <xsl:when test="$data-type='string'">
                <xsl:value-of select="$text_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='normalizedString'">
                <xsl:value-of select="$text_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='token'">
                <xsl:value-of select="$token_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='NMToken'">
                <xsl:value-of select="$NMToken_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='ID'">
                <xsl:value-of select="$ID_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='IDREF'">
                <xsl:value-of select="$IDREF_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='date'">
                <xsl:value-of select="$date_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='datetime'">
                <xsl:value-of select="$datetime_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='time'">
                <xsl:value-of select="$time_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='duration'">
                <xsl:value-of select="$duration_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='decimal'">
                <xsl:value-of select="$decimal_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='double'">
                <xsl:value-of select="$double_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='float'">
                <xsl:value-of select="$float_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='integer' or($data-type='int' )">
                <xsl:value-of select="$integer_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='byte'">
                <xsl:value-of select="$byte_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='boolean'">
                <xsl:value-of select="$boolean_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='long'">
                <xsl:value-of select="$long_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='short'">
                <xsl:value-of select="$short_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='list'">
                <xsl:value-of select="$list_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='NCName'">
                <xsl:value-of select="$NCName_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='QName'">
                <xsl:value-of select="$QName_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='any'">
                <xsl:value-of select="$any_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='anyAttribute'">
                <xsl:value-of select="$any_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='anyURI'">
                <xsl:value-of select="$uri_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='language'">
                <xsl:value-of select="$language_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='base64Binary'">
                <xsl:value-of select="$binary_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='hexBinary'">
                <xsl:value-of select="$binary_uuid"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$text_uuid"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="as-xsl:content">
        <xsl:param name="content"/>
        <xsl:if test="count($content/attribute) &gt; 0">
            <xsl:variable name="contentType">
                <xsl:choose>
                    <xsl:when test="$content/attribute[@NAME = 'list']">
                        <xsl:value-of select="$list_uuid"/>
                    </xsl:when>
                    <xsl:when test="$content/attribute[@NAME = 'number']">
                        <xsl:value-of select="$decimal_uuid"/>
                    </xsl:when>
                    <xsl:when test="$content/attribute[@NAME = 'datetype']">
                        <xsl:value-of select="$datetime_uuid"/>
                    </xsl:when>
                    <xsl:otherwise/>
                </xsl:choose>
            </xsl:variable>
            <uml:StructuralFeature.type>
                <uml:Class xmi.idref="{$contentType}"/>
            </uml:StructuralFeature.type>
        </xsl:if>
    </xsl:function>
    <xsl:function name="as-xsl:dbMapping">
        <xsl:param name="context"/>
        <xsl:param name="DBMapping"/>        
        <xsl:if test="count($DBMapping) &gt; 0">
            <node CREATED="{$EpochSeconds}" ID="ID_{generate-id($context)}"
                MODIFIED="{$EpochSeconds}" POSITION="right"
                STYLE="bubble"
                COLOR="#f05918"
                TEXT="{$DBMapping}">
                <edge WIDTH="4"/>
                <font BOLD="true" NAME="SansSerif" SIZE="12"/>
            </node>
        </xsl:if>
    </xsl:function>
    <xsl:function name="as-xsl:colorAttr">
        <xsl:param name="xpath"/>
        <xsl:param name="excluded"/>
        <xsl:param name="choice"/>
        <xsl:choose>
            <xsl:when test="not($excluded='')">
                <xsl:attribute name="COLOR">#ff6666</xsl:attribute>
            </xsl:when>                    
            <xsl:when test="count($descriptions/HasChanged[ends-with(@xpath,$xpath)]) &gt; 0">
                <xsl:attribute name="COLOR">#4fb658</xsl:attribute>
            </xsl:when>
            <xsl:when test="contains($details,'cardinality')">
                <xsl:if test="$choice != ''">
                    <xsl:attribute name="COLOR">#0007ff</xsl:attribute>
                </xsl:if>
            </xsl:when>
        </xsl:choose>
    </xsl:function>
    <xsl:function name="as-xsl:builtinAttr">
        <xsl:param name="xpath"/>
        <xsl:param name="excluded"/>
        <xsl:choose>
            <xsl:when test="count($descriptions/HasMapping[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="button_ok"/></xsl:when>
            <xsl:when test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:when>
            <xsl:when test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:when>
        </xsl:choose>
    </xsl:function>
    <xsl:function name="as-xsl:excluded">
        <xsl:param name="xpath"/>
        <xsl:value-of select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
    </xsl:function>
    <xsl:function name="as-xsl:cardinality">
        <xsl:param name="xpath"/>
        <xsl:variable name="result">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="$rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]"
                    >*</xsl:if>
                <xsl:if test="$rules/Rule[@type='optional'][ends-with(@xpath,$xpath)]"
                    >?</xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:value-of select="$result"/>
    </xsl:function>
    <xsl:function name="as-xsl:choice">
        <xsl:param name="name"/>
        <xsl:param name="parentXpath"/>        
        <xsl:variable name="result">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="$rules/Rule[@type='choice'][ends-with(@xpath,$parentXpath)][contains(@choices,concat('''',$name,''''))]"
                    >&lt; </xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:value-of select="$result"/>
    </xsl:function>
    <xsl:function name="as-xsl:getContent">
        <xsl:param name="xpath"/>
        <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
            <xsl:variable name="type"
                select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
            <xsl:variable name="mask"
                select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"/>
            <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
        </xsl:if>
    </xsl:function>
</xsl:stylesheet>
