<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xml="http://www.w3.org/XML/1998/namespace" xmlns:as="http://www.oasis-open.org/committees/cam"  xmlns:camed="http://jcam.org.uk/editor"
	xmlns:look="http://jcam.org.uk/LookupLists">
	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
		Authors: David RR Webber, September, 2009 -->
	<!--
		Purpose  : Utility for use with ERwin models. Parses input CAM template file and flags for removal the pieces that are model artifacts; renames model code detail parents.
		Inputs: CAM template.
		Original : 09/23/2009
		Initials : drrw
		Description : Parses input CAM template file and refactors into new CAM template.
		                     Uses default CAM template format, including any extensions.
			         W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
			         W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
			         W A R N I N G ! ! ! - Make sure to save template with Windows / Preferences / StructureAnnotations  set to save annotation in template TRUE!
		Updated  - Inits - Change
		03/03/10   drrw   Initial release 
		03/03/10   drrw   Added annotations item path change check
		03/05/10   drrw   _DETAIL handling
		03/06/10   drrw   _CODE and _COUNT checks
	-->
	<xsl:param name="inputFilePath"></xsl:param>

	<xsl:output method="xml" indent="yes"/>
	<xsl:variable name="xslt-version">1.03</xsl:variable>
	<xsl:variable name="nlchar" select="'&#xA;'"/>
	<xsl:variable name="debug">0</xsl:variable><!-- 0, 1, 2, 3, 4 or 5 -->
	<xsl:variable name="rootElement" select="as:CAM/as:AssemblyStructure/as:Structure/*[1]/name()"/>
	
	<xsl:variable name="namespaceLookup">
		<namespaces>
		<xsl:for-each select=".//namespace::*">
			<xsl:if test="not(name()='w2e')">
				<namespace name="{name()}" select="{.}"/>
			</xsl:if>
		</xsl:for-each>
		</namespaces>
	</xsl:variable>
	
	<xsl:variable name="itemAnnotations">
		<!-- xsl:message>anno inputFilePath: <xsl:value-of select="$inputFilePath"/></xsl:message -->
		<xsl:choose>
			<xsl:when test="//as:Extension"><xsl:copy-of select="//as:Extension"/></xsl:when>
			<xsl:when test="//as:Include/as:Extension"><xsl:copy-of select="//as:Include/as:Extension"/></xsl:when>
			<xsl:otherwise>
				<xsl:variable name="annoIncludes"><xsl:copy-of select="//as:include"/></xsl:variable>
				<xsl:for-each select="$annoIncludes/as:include">
					<xsl:variable name="filePath"><xsl:value-of select="if(contains($inputFilePath,'file://')) then $inputFilePath else concat('file://',$inputFilePath)"/><xsl:value-of select="."/></xsl:variable>
					<xsl:if test="contains(.,'.anno')"><xsl:copy-of select="document(replace(replace($filePath,'C:\\','\\'),'\\','/'))"/></xsl:if>
				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name="refactoredStructure">
		<xsl:apply-templates select="//as:AssemblyStructure/as:Structure" mode="structure"/>
	</xsl:variable>
	
	<xsl:variable name="newRules">
		<xsl:apply-templates select="$refactoredStructure/as:Structure" mode="structureRules"/>
	</xsl:variable>
	
	<xsl:variable name="newAnno">
		<xsl:apply-templates select="$refactoredStructure/as:Structure" mode="structureAnno"/>
	</xsl:variable>
	
	<xsl:strip-space elements="*"/>
	
	<xsl:template match="/">
		<xsl:variable name="templateNS" select=".//namespace::*"/>
		<xsl:message>Annotation checking starting <xsl:value-of select="dateTime(current-date(),current-time())"/></xsl:message>
		<xsl:message>Version: <xsl:value-of select="$xslt-version"/></xsl:message>
		<xsl:message>From template structure: <xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/></xsl:message>
		
		<as:CAM xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:xml="http://www.w3.org/XML/1998/namespace" CAMlevel="{as:CAM/@CAMlevel}" version="{as:CAM/@version}">
			<xsl:for-each select=".//namespace::*">
				<xsl:if test="not(name()='w2e')">
					<xsl:namespace name="{name()}" select="."/>
				</xsl:if>
			</xsl:for-each>

<xsl:comment xml:space="preserve">
	CAM Annotations checker
	Version: <xsl:value-of select="$xslt-version"/>
	Date: <xsl:value-of select="dateTime(current-date(),current-time())"/>
	From template structure: <xsl:value-of select="//as:AssemblyStructure/as:Structure/@ID"/>

</xsl:comment>

			<xsl:message>Writing structure</xsl:message>
			<xsl:message> (path trace diagnostics)</xsl:message>
			<xsl:apply-templates select="//as:Header"/>
			<xsl:variable name="parsedStructure">
				<xsl:element name="as:AssemblyStructure"><!-- output the expanded Structure details -->
					<xsl:for-each select="//as:AssemblyStructure/@*">
						<xsl:attribute name="{name()}" select="."/>
					</xsl:for-each>
					<xsl:copy-of select="$refactoredStructure"/>
				</xsl:element>
			</xsl:variable>
			<xsl:apply-templates mode="outputStructure" select="$parsedStructure"/>
			<xsl:message>structure completed</xsl:message>
			<xsl:message>Writing rules</xsl:message>
			<xsl:apply-templates select="//as:BusinessUseContext"/>
			<xsl:message>completed rules</xsl:message>
			<!-- xsl:message>annotations: [<xsl:copy-of select="$itemAnnotations"/>]</xsl:message -->
			
			<xsl:message>Writing annotations</xsl:message>
			<xsl:choose>
				<xsl:when test="//as:Extension"><xsl:apply-templates select="//as:Extension"/></xsl:when>
				<xsl:when test="//as:include"><xsl:apply-templates select="//as:include"/></xsl:when>
				<xsl:otherwise>
					<as:Extension name="uk.org.jcam.camed.extensions.StructureAnnotations">

						<xsl:for-each select="distinct-values($newAnno/as:MoreAnno/camed:annotation/@item)">
							<xsl:variable name="thisAnnoItem" select="."/>
							<xsl:copy-of select="$newAnno/as:MoreAnno/camed:annotation[@item = $thisAnnoItem]"/>
						</xsl:for-each>
					</as:Extension>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:message>Total annotations added: <xsl:value-of select="count(distinct-values($newAnno/as:MoreAnno/camed:annotation/@item))"/></xsl:message>
			<xsl:message>Completed annotations</xsl:message>
			<!-- xsl:apply-templates select="//as:include"/ --><!-- disable for now, CAMeditor will recreate includes on save -->			
		</as:CAM>
		<xsl:message>CAM annotations checker finished processing.</xsl:message>
	</xsl:template>

	<xsl:template match="as:Header">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:AssemblyStructure">
		<xsl:copy>
			<xsl:apply-templates select="as:Structure"/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="as:BusinessUseContext">
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="as:Extension">
		<as:Extension name="{@name}">
			<xsl:apply-templates/>
			<xsl:copy-of select="$newAnno/as:MoreAnno/*"/>
		</as:Extension>
	</xsl:template>
	
	<xsl:template match="camed:annotation">
		<camed:annotation item="{@item}">
			<xsl:apply-templates/>
		</camed:annotation>
	</xsl:template>
	
	<xsl:template match="camed:documentation">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="look:LookupList">
		<xsl:copy>
			<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	
	<xsl:template match="look:item">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:Structure">
		<as:Structure ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates mode="structure"/>
		</as:Structure>
	</xsl:template>
	
	<xsl:template match="as:Structure" mode="structureRules">
		<as:MoreRules ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates select="./*" mode="structureRules"/>
		</as:MoreRules>
	</xsl:template>

	<xsl:template match="as:Structure" mode="structureAnno">
		<as:MoreAnno ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<xsl:apply-templates select="./*" mode="structureAnno"/>
		</as:MoreAnno>
	</xsl:template>

	<xsl:template match="as:Rules">
		<as:Rules>
			<as:default>
				<xsl:apply-templates select="as:default"/>
			</as:default>
			<xsl:if test="as:context">
			<as:context>
				<xsl:attribute name="condition"><xsl:value-of select="as:context/@condition"/></xsl:attribute>
				<xsl:apply-templates select="as:context"/>
			</as:context>
			</xsl:if>
		</as:Rules>
	</xsl:template>
	
	<xsl:template match="as:default">
		<as:context>
			<xsl:apply-templates select="as:context"/>
			
			<!-- Now add exclude rules from structure pass -->
			<xsl:variable name="excludeRules" select="$refactoredStructure//as:constraint"/>
			<xsl:for-each select="distinct-values($excludeRules//./@action)">
				<xsl:variable name="thisAction" select="."/>
				<xsl:variable name="actionRules" select="$excludeRules//.[@action = $thisAction]"/>
				<xsl:apply-templates select="$actionRules[1]"/>
			</xsl:for-each>
			
			<!-- xsl:message> total rules added: <xsl:value-of select="count($uniqueRules)"/></xsl:message -->
		</as:context>
	</xsl:template>
	
	<xsl:template match="as:context">
		<xsl:apply-templates mode="checkRules"/>
	</xsl:template>
	
	<xsl:template match="as:constraint">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	<xsl:template match="as:constraint" mode="outputStructure"/>
	
	<xsl:template match="as:constraint" mode="checkRules">
		<!-- check to see if parent node renamed -->
		<xsl:choose>
			<xsl:when test="contains(./@action,'_CODE/') or(contains(./@action,'_COUNT'))">
				<xsl:variable name="parentPath" select="concat(substring-before(substring-after(./@action,'//'),'/'),'_DETAIL')"/>
				<xsl:variable name="matchNode" select="count($refactoredStructure/as:Structure/AllRootLevelTypeDefinitions/node()[name()=$parentPath])"/>
				<xsl:choose>
					<xsl:when test="$matchNode &gt; 0">
						<xsl:element name="as:constraint">
							<xsl:if test="@condition"><xsl:attribute name="condition" select="@condition"/></xsl:if>
							<xsl:attribute name="action" select="replace(@action,replace($parentPath,'_DETAIL',''),$parentPath)"/>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise><xsl:apply-templates select="."/></xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise><xsl:apply-templates select="."/></xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy>
			<xsl:if test="string-length($include) &gt; 0">
				<xsl:attribute name="as:include" select="$include"/>
			</xsl:if>
			<xsl:apply-templates mode="start"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="@*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:include" mode="start">
		<!-- If there are any includes expand them to inline structure components (TBD the CAMeditor may already do this in Java -->
		<!-- TBD This part may need attention -->
		<xsl:variable name="filePath"><xsl:value-of select="if(contains($inputFilePath,'file://')) then $inputFilePath else concat('file://',$inputFilePath)"/>/<xsl:value-of select="."/></xsl:variable>
		<xsl:variable name="include" select="document($filePath)"/>
		<xsl:apply-templates select="$include/*" mode="start">
			<xsl:with-param name="include" select="."/>
		</xsl:apply-templates>		
		
	</xsl:template>
	
	<xsl:template match="as:include">
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="*" mode="structure">
		<xsl:param name="parent"/>
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="parentItem" select="if(../name()='as:Structure') then '' else ../name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parentPath" select="replace(concat('/',$parentItem,$contextMatch,'/'),'//','/')"/>
		<!-- find virtual parent path, this is the parent in the dictionary that precedes this item to be expanded -->
		<xsl:if test="$debug &gt; 0"><xsl:message>- <xsl:value-of select="$thisItem"/>[<xsl:value-of select="$parentItem"/>][<xsl:value-of select="$rootElement"/>]</xsl:message></xsl:if>
		
		<xsl:variable name="noNSname" select="if(contains($thisItem,':')) then substring-after($thisItem,':') else $thisItem"/>
		<xsl:variable name="noNSmatch" select="concat('/',$noNSname)"/>
		
		<xsl:variable name="rootNode" select="if(parent::*/name()='AllRootLevelTypeDefinitions') then true() else false()"/>
		<xsl:if test="$rootNode = true()"><xsl:message> - parent <xsl:value-of select="name()"/></xsl:message></xsl:if>
		
		
		<!-- Create structure header, or process child nodes  -->
		<xsl:choose>
			<xsl:when test="name()='as:Structure'">
				<xsl:variable name="actualRootNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:if test="$debug &gt; 1"><xsl:message>as:Structure root: [<xsl:value-of select="name()"/>][<xsl:copy-of select="."/>]</xsl:message></xsl:if>
				<xsl:element name="{name()}" namespace="{$actualRootNS}">
					<xsl:apply-templates select="attribute::*" mode="insert"/>
					<xsl:message>- structure of: <xsl:value-of select="./*/name()"/></xsl:message>
					<xsl:variable name="newStructure" select="./*"/>
					<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="./*/name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
					<xsl:element name="{./*/name()}" namespace="{$actualNS}"><!-- Insert root element, then process children -->
						<xsl:apply-templates select="$newStructure/attribute::*" mode="insert"/>
						<xsl:apply-templates mode="structure" select="$newStructure/./*"/>
					</xsl:element>
				</xsl:element>
			</xsl:when>
			
			<xsl:otherwise>
				<xsl:variable name="actualNS"><xsl:call-template name="nsResolver"><xsl:with-param name="itemName" select="name()"/><xsl:with-param name="ns" select="namespace-uri()"/></xsl:call-template></xsl:variable>
				<xsl:if test="$debug &gt; 1"><xsl:message>structure here: [<xsl:value-of select="name()"/>][<xsl:copy-of select="."/>]</xsl:message></xsl:if>
				
				<!-- If element contains child of same name, then rename parent with _DETAIL -->
				
				<xsl:variable name="nodeName" select="name()"/>
				
				<xsl:element name="{$nodeName}" namespace="{$actualNS}">
					<xsl:apply-templates select="attribute::*" mode="insert"/>
					<xsl:if test="not(exists(*)) and string-length(text()) &gt; 0"><xsl:value-of select="text()"/></xsl:if><!-- insert default value text -->
					<xsl:if test="as:include">
						<xsl:apply-templates mode="include"/>
					</xsl:if>
					<xsl:if test="$rootNode = true()"><xsl:message> - item <xsl:if test="count(child::*) &gt; 0"> - <xsl:value-of select="count(child::*)"/></xsl:if> : <xsl:value-of select="name()"/></xsl:message></xsl:if>
								
					<xsl:apply-templates mode="structure">
						<xsl:with-param name="parent" select="$nodeName"/>
					</xsl:apply-templates>
				</xsl:element>
				
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="text()" mode="structure"/>
	
	<xsl:template match="text()" mode="structureRules"/>
	
	<xsl:template match="@*" mode="insert">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:attribute name="{name()}" namespace="{$namespace-uri}"><xsl:value-of select="."/></xsl:attribute>
	</xsl:template>

	<xsl:template match="*|@*|text()">
	</xsl:template>

	<xsl:template match="*" mode="rule">
		<xsl:param name="action"/>

		<xsl:copy-of select="."/>
		
	</xsl:template>
	
	<!-- These templates are from original code -->
	
	<xsl:template match="as:constraint|as:context|@*" mode="startRules">
		<xsl:variable name="action" select="@action"/>
		<xsl:choose>
			<xsl:when test="contains($action,'applyTemplate')">
				<xsl:variable name="templateName" select="if (contains(substring-after($action,','),',')) then substring-before(substring-after($action,','),',') else substring-before(substring-after($action,','),')')"/>
				<xsl:variable name="templateFileName" select="concat('file://',$inputFilePath,'/',$templateName)"/>
				<!-- xsl:message>templateFileName=<xsl:value-of select="$templateFileName"/></xsl:message -->
				<xsl:variable name="template" select="document($templateFileName)"/>
				<xsl:variable name="applyTemplateRules" select="$template//as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint"/>
				<xsl:apply-templates select="$applyTemplateRules" mode="start"/>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="*" mode="structureRules">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parent" select="./../name()"/>
		
		<!-- Find the names of the child items of this node -->
		<xsl:variable name="theseChildrenMembers" select="child::node()"/>
		<xsl:variable name="ChildElementNames">
			<xsl:for-each select="$theseChildrenMembers/.">|<xsl:value-of select="name()"/>|</xsl:for-each>
		</xsl:variable>
		
		<xsl:variable name="childNodesCount" select="count(tokenize($ChildElementNames,'\|\|'))"/>
		
		<!-- xsl:if test="not($ChildElementNames='||')"><xsl:message>child nodes: [<xsl:value-of select="$childNodesCount"/>] <xsl:value-of select="concat($parent,$contextMatch)"/> - <xsl:value-of select="$ChildElementNames"/></xsl:message></xsl:if -->

		<!-- Choose to lookup item structure in dictionary, or simply skip  -->
		<!-- xsl:message> - <xsl:value-of select="concat('/',$parent,'/',$thisItem)"/> | <xsl:value-of select="$namespacesList"/> | <xsl:value-of select="$thisNS"/></xsl:message -->
		
		<xsl:apply-templates mode="structureRules"/>
		
	</xsl:template>
	
	<xsl:template match="@*" mode="structureRules">
		<xsl:param name="context"/>
		<xsl:variable name="findAttrPath" select="concat($context,name())"/>
		<xsl:for-each select="//Property[ends-with(@context,$findAttrPath)][1]">
			<xsl:if test="contains(./@cardinality,'optional')">
				<as:constraint action="{concat('makeOptional(/',if(contains(./@context,'/@')) then ./@context else replace(./@context,'@','/@'),')')}"/>
			</xsl:if>
			<xsl:if test="contains(./@cardinality,'repeatable')">
				<as:constraint action="{concat('makeRepeatable(/',if(contains(./@context,'/@')) then ./@context else replace(./@context,'@','/@'),')')}"/>
			</xsl:if>
			<xsl:call-template name="setFacetRules">
				<xsl:with-param name="type" select="@type"/>
				<xsl:with-param name="mask" select="@mask"/>
				<xsl:with-param name="path" select="concat('/',@context)"/>
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="setFacetRules">
		<xsl:param name="type"/>
		<xsl:param name="mask"/>
		<xsl:param name="path"/>
		<xsl:variable name="thisPath" select="if(contains($path,'/@')) then $path else replace($path,'@','/@')"/>
		<xsl:choose>
			<xsl:when test="$type='ID'"><as:constraint action="{concat('datatype(',$thisPath,',ID)')}"/></xsl:when>
			<xsl:when test="$type='IDREF'"><as:constraint action="{concat('datatype(',$thisPath,',IDREF)')}"/></xsl:when>
			<xsl:when test="$type='IDREFS'"><as:constraint action="{concat('datatype(',$thisPath,',IDREFS)')}"/></xsl:when>
			<xsl:when test="$type='any'"><as:constraint action="{concat('datatype(',$thisPath,',any)')}"/></xsl:when>
			<xsl:when test="$type='anyURI'"><as:constraint action="{concat('datatype(',$thisPath,',anyURI)')}"/></xsl:when>
			<xsl:when test="$type='base64Binary'"><as:constraint action="{concat('datatype(',$thisPath,',base64Binary)')}"/></xsl:when>
			<xsl:when test="$type='anyAttribute'"><as:constraint action="{concat('datatype(',$thisPath,',anyAttribute)')}"/></xsl:when>
			<xsl:when test="$type='boolean'"><as:constraint action="{concat('datatype(',$thisPath,',boolean)')}"/></xsl:when>
			<xsl:when test="$type='integer'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='decimal'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='float'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='unsignedInt'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',if($mask='') then '########' else $mask,')')}"/></xsl:when>
			<xsl:when test="$type='unsignedLong'"><as:constraint action="{concat('setNumberMask(',$thisPath,',',if($mask='') then '############' else $mask,')')}"/></xsl:when>
			<xsl:when test="$type='date'"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='time'"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:when test="$type='list'"><as:constraint action="{concat('restrictValues(',$thisPath,',''',replace($mask,',','''|'''),''')')}"/></xsl:when>
			<xsl:when test="contains($mask,'|')"><as:constraint action="{concat('restrictValues(',$thisPath,',''',replace($mask,'\|','''|'''),''')')}"/></xsl:when>
			<xsl:when test="$type='datetime'"><as:constraint action="{concat('setDateMask(',$thisPath,',',$mask,')')}"/></xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="setTypeSample">
		<xsl:param name="type"/>
		<xsl:param name="mask"/>
		<xsl:choose>
			<xsl:when test="$type=''"/>
			<xsl:when test="$type='date'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='datetime'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='time'">%<xsl:value-of select="$mask"/>%</xsl:when>
			<xsl:when test="$type='integer'">%1234%</xsl:when>
			<xsl:when test="$type='decimal'">%654.10%</xsl:when>
			<xsl:when test="$type='float'">%7654.0001%</xsl:when>
			<xsl:when test="$type='anyURI'">%http://www.camtoolkit.com%</xsl:when>
			<xsl:otherwise>%<xsl:value-of select="$type"/>%</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="nsResolver">
		<xsl:param name="itemName"/>
		<xsl:param name="ns"/>
		<xsl:choose>
			<xsl:when test="count(tokenize($ns,'http:')) &gt; 1">
				<xsl:choose>
					<xsl:when test="contains($itemName,':')">
						<xsl:variable name="thisNS" select="substring-before($itemName,':')"/>
						<xsl:value-of select="$namespaceLookup/namespaces/namespace[@name=$thisNS][1]/@select[1]"/>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="$ns=''">
				<xsl:choose>
					<xsl:when test="contains($itemName,':')">
						<xsl:variable name="thisNS" select="substring-before($itemName,':')"/>
						<xsl:variable name="matchNS" select="$namespaceLookup/namespaces/namespace[@name=$thisNS][1]/@select[1]"/>
						<xsl:choose>
							<xsl:when test="string-length($matchNS) &lt; 1"><xsl:value-of select="concat('http://jcam.org.uk/ns/',$thisNS)"/></xsl:when>
							<xsl:otherwise><xsl:value-of select="$matchNS"/></xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
				
			</xsl:when>
			<xsl:otherwise><xsl:value-of select="$ns"/></xsl:otherwise>
		</xsl:choose>	
	</xsl:template>
	
	<xsl:template match="*" mode="structureAnno">
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="thisNS" select="substring-before(name(),':')"/>
		<xsl:variable name="thisItem" select="name()"/>
		<xsl:variable name="contextMatch" select="concat('/',$thisItem)"/>
		<xsl:variable name="parent" select="./../name()"/>
		
		<!-- Make sure there is an annotation, if not then look it up  -->						
			
		<xsl:variable name="lookupPath" select="concat('//',parent::*/name(),'/',$thisItem)"/>	
		
		<xsl:choose>
			<xsl:when test="parent::*/name() = 'AllRootLevelTypeDefinitions'"><xsl:message> - <xsl:value-of select="$thisItem"/></xsl:message></xsl:when>
			<xsl:when test="$itemAnnotations//camed:annotation[@item = $lookupPath]"/>
			<xsl:otherwise><!-- get annotation to add -->
				<xsl:variable name="itemAnno" select="$itemAnnotations//camed:annotation[ends-with(@item,$contextMatch)][1]"/>
				<xsl:choose>
					<xsl:when test="count($itemAnno) &gt; 0">
						<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
							<xsl:attribute name="item" select="$lookupPath"/>
							<xsl:apply-templates select="$itemAnno[1]//camed:documentation"/>
						</xsl:element>
					</xsl:when>
					<xsl:when test="ends-with($thisItem,'CodeDetail')">
						<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
							<xsl:attribute name="item" select="$lookupPath"/>
							<camed:documentation type="Definition">Details of the code and attributes for <xsl:value-of select="replace($thisItem,'CodeDetail','Code')"/>.</camed:documentation>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise><xsl:message> - MISSING ANNO!: <xsl:value-of select="$lookupPath"/></xsl:message></xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
		
		<xsl:apply-templates mode="structureAnno"/>
		
	</xsl:template>
	
	<xsl:template match="*" mode="importAnno">
		<xsl:param name="this"/>
		<xsl:param name="parent"/>
		<xsl:param name="rest"/>
		<xsl:variable name="thisItem" select="$this/./@item"/>
		<xsl:variable name="thisContext" select="$this/./@context"/>		
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<xsl:variable name="more" select="count($rest/Property)"/>
		<!-- xsl:message>anno: [<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$more"></xsl:value-of>]</xsl:message -->
		<xsl:variable name="parentContext" select="concat($this/./@context,'/')"/>
		<xsl:variable name="shortContext">
			<xsl:call-template name="shortContext">
				<xsl:with-param name="thisContext" select="$thisContext"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:choose>
			<!-- xsl:when test="$this/./@CCTS ='ASBIE'"/ --><!-- ignore attributes -->
			<xsl:when test="$more &gt; 0">
				<!-- Now insert the annotation from the definition details -->
				
				<xsl:for-each select="$rest/Property">
					<!-- xsl:message>[<xsl:value-of select="$thisItem"/>][<xsl:value-of select="$parentContext"/>][<xsl:value-of select="@context"/>]</xsl:message -->
					<!-- Now see if there are children of this element -->
					<xsl:variable name="chkContext" select="concat(@context,'/')"/>
					<xsl:variable name="itemContext" select="substring-after(@context,$parentContext)"/>
					<xsl:variable name="next-namespace-uri" select="namespace-uri()"/>
					<xsl:variable name="children" select="count($rest/Property[starts-with(@context,$chkContext)])"/>
					<xsl:choose>
						<xsl:when test="$children &gt; 0">
							<!-- Process parent and children -->
							<xsl:variable name="importRest">
								<xsl:for-each select="$rest/Property[contains(@context,$chkContext)]">
									<xsl:copy-of select="."/>
								</xsl:for-each>
							</xsl:variable>
							<xsl:variable name="uniqueAnnoList" select="distinct-values($importRest/Property/@context)"/>
							<xsl:variable name="importRestUnique">
								<xsl:for-each select="$uniqueAnnoList">
									<xsl:variable name="thisContextItem" select="."/>
									<xsl:copy-of select="$importRest/Property[@context=$thisContextItem][1]"/>
								</xsl:for-each>
							</xsl:variable>
							<xsl:apply-templates select="." mode="importAnno">
								<xsl:with-param name="this" select="."/>
								<xsl:with-param name="parent" select="$thisItem"/>
								<xsl:with-param name="rest" select="$importRestUnique"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="contains($itemContext,'/')"><!-- Ignore child of parent item --></xsl:when>
						<xsl:otherwise><!-- Process this singleton element -->
							<xsl:if test="string-length(./@definition) &gt; 0">
								<xsl:variable name="shortContext">
									<xsl:call-template name="shortContext">
										<xsl:with-param name="thisContext" select="concat('/',./@context)"/>
										<xsl:with-param name="parent" select="$parent"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
									<xsl:attribute name="item" select="$shortContext"/>
									<camed:documentation type="Definition"><xsl:value-of select="./@definition"/></camed:documentation>
									<camed:documentation type="Expanded">From <xsl:value-of select="./@item"/> dictionary definition.</camed:documentation>
								</xsl:element>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
				
			</xsl:when>
			<xsl:otherwise>
				<!-- Process this singleton element -->
					<xsl:variable name="shortContext">
						<xsl:call-template name="shortContext">
							<xsl:with-param name="thisContext" select="concat('/',$this/./@context)"/>
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:element name="camed:annotation" namespace="http://jcam.org.uk/editor">
						<xsl:attribute name="item" select="$shortContext"/>
						<xsl:if test="string-length($this/./@definition) &gt; 0"><camed:documentation type="Definition"><xsl:value-of select="$this/./@definition"/></camed:documentation></xsl:if>
						<camed:documentation type="Expanded">From <xsl:value-of select="$this/./@item"/> dictionary definition.</camed:documentation>
					</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="shortContext">
		<xsl:param name="thisContext"/>
		<xsl:param name="parent"/>
		<xsl:variable name="contextPathNodes" select="tokenize($thisContext,'/')"/>
		<xsl:variable name="contextPathCount" select="count($contextPathNodes)"/>
		<xsl:variable name="shortContext">
			<xsl:choose>
				<xsl:when test="$contextPathCount &gt; 3"><xsl:value-of select="concat('//',$contextPathNodes[$contextPathCount - 1],'/',$contextPathNodes[$contextPathCount])"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat('//',replace(concat($parent,$thisContext),'//','/'))"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="if(contains($shortContext,'@') and not(contains($shortContext,'/@'))) then replace($shortContext,'@','/@') else $shortContext"/>
	</xsl:template>
	
	<xsl:template match="*" mode="outputStructure">
		<xsl:element name="{name()}">
			<xsl:apply-templates mode="outputStructure" select="@*"/>
			<xsl:apply-templates mode="outputStructure"/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="@*" mode="outputStructure">
		<xsl:copy-of select="."/>
	</xsl:template>

</xsl:stylesheet>