<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:cam2func="http://jcam.org.uk/editor/function" xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:dblook="http://jcam.org.uk/DbLookupLists"
    exclude-result-prefixes="xs xsd as camed cam2func look dblook" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, June, 2008 -->

    <!-- 

Purpose : Take a CAM template format in CXF format and produce diagnostics, NDR evaluation and validation check report in HTML.
Original : 06/21/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
How to use :  Source input is CAM template in CXF format. Returns HTML page of summary of analysis.
Updated - Inits - Change 
06/21/08 drrw Initial release.
06/22/08 drrw Add setLength check
06/23/08 drrw Re-factor into two sections - CAM and XSD related
06/24/08 mmer Add selected validation and duplicate rule reporting
06/25/08 drrw Postponed xs:anyType warning
09/05/08 drrw Message text cleanup
12/12/08 drrw Add warning check for fixed values
02/18/09 drrw Hint count defaulting to 1
02/18/09 drrw Add new check for namespace issues
04/10/09 drrw Add warning for strings with length limits except number, id or code
04/10/09 drrw Add warning for codes and indicators with no restrictValues
04/10/09 drrw Add warning for date or time item with no mask
04/10/09 drrw Correct HTML output - add header, fix namespace use and ul formatting
04/10/09 drrw Improve section heading text and make list item headers bold font
04/11/09 drrw Refine XPath rule matching for no restrictValues and setDateMask
04/13/09 drrw Scan children of element for no restrictValues and setDateMask detection
04/18/09 drrw Make the NDR functions an xslt include file
04/19/09 drrw Improve NDR diagnostics by namespace
04/21/09 drrw Relax NDR rule on choice, it is allowed in exchange and extension schemas
04/22/09 drrw Relax NDR rules for items in W3C or OASIS domains (parameter controlled)
04/27/09 drrw Add display of NDR representation terms
04/29/09 drrw Fix html <li> formatting issue for lists
05/04/09 drrw Correct inverse logic on domain to ignore check
05/14/09 drrw Change diagnostics to say Evaluation instead of Validator
05/28/09 drrw Only show missing date typing from elements when child date element not present
08/18/09 drrw Mistakes exclude in restrictValues for exclude rule on attributes
12/05/09 drrw Suppress term hints when no term error
12/10/09 drrw Add Indicator to flag and code check
01/12/10 drrw Skip words validate, consolidate and update in date checking
01/22/10 drrw Add spelling review section 
01/27/10 drrw Skip words AgendaText  (date)
02/01/10 drrw Ignore ZIP code items for rules check
03/30/10 drrw Produce statistics for uniques
06/15/10 drrw Add html line separator to bottom
08/16/10 drrw Update logic for latest annotation changes
08/17/10 drrw Enhance evaluator to catch missing setNumberMask on numeric 
08/17/10 drrw Enhance evaluator to catch conflicting rule use (e.g. mandatory and optional on same node) 
08/17/10 drrw Check for choice() with only one node 
08/24/10 drrw Refactor numeric rule check
12/07/10 drrw Ignore Candidate as word matching Date check
12/07/10 drrw Refine NDR name checking rules for NIEM structure attributes
12/30/10 drrw Refine Reference Terms and Namespace association checking
12/31/10 drrw Add mode detection for name checking mode - NIEM / Camel Upper / Camel Lower
01/02/11 drrw Enhance upper/lower case name logic
01/02/11 drrw Enhance term representation checks by namespace
01/12/11 drrw Add check that lookups have codelist table values
04/28/11 drrw Give warning for duplicate restrictValues, not error
06/14/11 drrw Fix false positive on fixed values with trailing space
06/15/11 drrw Fix false match on numeric type checking for ValueText and TelephoneNumber
07/09/11 drrw Fix false match on CODE and text
07/09/11 drrw Relax code check if item has a set___Mask() restriction
02/08/12 drrw Relax check on SQL lookups
02/08/12 drrw  Prevent false positive report of code items with datatype(boolean)
04/21/12 drrw  Add Image, and Total to reference terms
04/21/12 drrw  Adjust name checking by removing namespace prefix
04/22/12 drrw  Flag content rules that are on parent elements with children (mixed content)
04/23/12 drrw  setChoice fix to skip in mixed content check - use action contains Mask( instead.
04/23/12 drrw  Add setLength and setNumberRange to mixed content check
04/26/12 drrw  Add length to report of items with setLength restriction
06/25/12 drrw  Treat datatype(anyAttribute) not as mixed content rule
06/25/12 drrw  Add details to report of setLimit / nillable - show factors
06/27/12 drrw  Fine tune setLength restriction check to ignorefixed length code values
09/17/12 drrw  Make setLength() processing fail safe
09/28/12 drrw  Improve nillable rules reporting - group by optional and required
09/28/12 drrw  Report items that are nillable, optional and repeatable
09/30/12 drrw  Evaluator flags date field as missing setDateMask rule - when it in fact has a datatype(dateTime) rule
09/30/12 drrw  Reference terms added Context 
10/03/12 drrw  Refactor date / time rule checking
10/03/12 drrw  Improve number checking to include datatype()
03/20/13 drrw  Add XML and Score to reference terms

??/20/13 drrw  Evaluate DBMapping statements to flag duplicate table/column mapping statements pointing to different parent XPath locations - (this causes duplicate SQL reads).
??/20/13 drrw  Add configurable common abbreviations list to reference terms - e.g. SSN, et al so those items do not fail reference terms checking.
??/20/13 drrw  Currently use of plural - numbers - values - amounts not detected - along with child elements as components
??/20/13 drrw  E.g. flagging LineItemsAmounts - that has children of Currency + ItemAmount; Need to look for child elements or attributes - in same way that the Date handling does.

    -->

    <!-- NDR rule implementation documentation cross-reference
        
        Currently checked and or supported from NIEM NDR rules: 
        
        1.1_rule_6-7, 1.1_rule_6-8, 1.3_rule_6-10, 1.3_rule_6-11, 1.3_rule_6-35,
        1.3_rule_6-37, 1.3_rule_6-39, 1.3_rule_6-41, 1.3_rule_6-46,
        1.3_rule_6-47, 1.3_rule_6-49, 1.3_rule_9-5, 1.3_rule_9-6, 1.3_rule_9-19,
        1.3_rule_9-22 
    -->

    <xsl:param name="ignoreNillableIn">false</xsl:param>
    <xsl:param name="ignoreSetLengthIn">false</xsl:param>
    <xsl:param name="ignoreSetLimitIn">false</xsl:param>
    <xsl:param name="ignoreFixedValues">false</xsl:param>
    <xsl:param name="ignoreNS">false</xsl:param>
    <xsl:param name="ignoreNameRuleType">false</xsl:param>
    <xsl:param name="ignoreDomainsNDR">false</xsl:param>
    <!-- xsl:param name="ignoreAnyType">false</xsl:param -->
    <xsl:output indent="yes"/>

    <xsl:variable name="version">1.45</xsl:variable>
    <xsl:variable name="nbsp">
        <xsl:text disable-output-escaping="yes">&#160;&#160;</xsl:text>
    </xsl:variable>

    <xsl:variable name="rules" select="/"/>
    <!-- Used by NDR functions -->

    <xsl:include href="cam-NDR-functions.xsl"/>
    <!-- xsl:include href="cam-NDR-terms.xsl"/ PENDING -->

    <xsl:variable name="allElements">
        <allElements>
            <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/as:Element/*"
                mode="list">
                <xsl:with-param name="type">element</xsl:with-param>
            </xsl:apply-templates>
        </allElements>
    </xsl:variable>
    <xsl:variable name="allAttributes">
        <allAttributes>
            <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/as:Element/*"
                mode="list">
                <xsl:with-param name="type">attribute</xsl:with-param>
            </xsl:apply-templates>
        </allAttributes>
    </xsl:variable>

    <xsl:variable name="namespaceDetail">
        <xsl:for-each select="as:CAM/as:Namespaces/as:namespace">
            <xsl:if test="contains(.,'niem')">NIEM|</xsl:if>
            <xsl:if test="contains(.,'urn:oasis')">OASIS|</xsl:if>
            <xsl:if test="contains(.,'niem/structures')">structure|</xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="namesDetail">
        <xsl:apply-templates select="$allElements/*" mode="names"/>
    </xsl:variable>

    <xsl:variable name="camelCaseMode">
        <xsl:choose>
            <xsl:when test="contains($namespaceDetail,'NIEM|')">NIEM</xsl:when>
            <xsl:when test="lower-case($namesDetail) = $namesDetail">LowerCase</xsl:when>
            <xsl:otherwise>UpperCase</xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="lookupTables" select="/as:CAM/as:Extension"/>

    <xsl:template match="/">
        <xsl:if test="count(//as:AssemblyStructure/as:Structure/as:Element)=0">
            <html>
                <body>
                    <h1>Input file is not valid CAM internal format (.cxf)</h1>
                </body>
            </html>
            <xsl:message terminate="yes">Input file is not valid CAM internal format
                (.cxf)</xsl:message>
        </xsl:if>
        <html>
            <head>
                <title>CAM Template Evaluation</title>
            </head>
            <body>
                <h2>CAM Template Evaluation</h2>
                <h4>Version <xsl:value-of select="$version"/></h4>
                <h4>CAM Template HEADER information:</h4>
                <table>
                    <tr>
                        <td>Description: </td>
                        <td>
                            <xsl:value-of select="//as:Header/as:Description"/>
                        </td>
                    </tr>
                    <tr>
                        <td>Owner: </td>
                        <td>
                            <xsl:value-of select="//as:Header/as:Owner"/>
                        </td>
                    </tr>
                    <tr>
                        <td>Date: </td>
                        <td>
                            <xsl:value-of select="//as:Header/as:DateTime"/>
                        </td>
                    </tr>
                    <tr>
                        <td>Version: </td>
                        <td>
                            <xsl:value-of select="//as:Header/as:Version"/>
                        </td>
                    </tr>
                </table>
                <ul>
                    <li>Total number of Elements: <xsl:value-of select="count(//as:Element)"/></li>
                    <li>Total number of Attributes: <xsl:value-of select="count(//as:Attribute)"
                        /></li>
                    <li>Total number of constraint Rules: <xsl:value-of
                            select="count(//as:Rule/as:constraint)"/></li>
                    <li>Annotations: <xsl:value-of
                            select="count(//as:Element/camed:annotation/camed:documentation[@type='Definition'])"
                        /></li>
                    <li>Hints: <xsl:value-of
                            select="count(//as:Element/camed:annotation/camed:documentation[@type='Value'])"
                        /></li>
                    <li>Excluded Elements: <xsl:value-of
                            select="count(//as:Element/as:Rule/as:constraint[starts-with(@action,'exclude')])"
                        /></li>
                    <li>Excluded Attributes: <xsl:value-of
                            select="count(//as:Attribute/as:Rule/as:constraint[starts-with(@action,'exclude')])"
                        /></li>
                </ul>
                <ul>
                    <li>Unique Elements count: <xsl:value-of
                            select="count(distinct-values(//as:Element/@name))"/></li>
                    <li>Unique Attributes count: <xsl:value-of
                            select="count(distinct-values(//as:Attribute/@name))"/></li>
                    <li>Unique constraint Rules count: <xsl:value-of
                            select="count(distinct-values(//as:Rule/as:constraint/concat(@xpath,@action)))"
                        /></li>
                    <li>Names mode: <xsl:value-of select="$camelCaseMode"/></li>
                </ul>
                <p/>
                <h3>RULES INTEGRITY:</h3>

                <ul>
                    <xsl:variable name="chkRules">
                        <xsl:call-template name="checkRules">
                            <xsl:with-param name="rules" select="//as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="count($chkRules/li) &gt; 0">
                            <xsl:for-each select="distinct-values($chkRules/li)">
                                <li style="color:red">
                                    <xsl:copy-of select="."/>
                                </li>
                            </xsl:for-each>
                        </xsl:when>
                        <xsl:otherwise>
                            <li>No problems found</li>
                        </xsl:otherwise>
                    </xsl:choose>
                </ul>
                <p/>
                <h3>NAMING AND DESIGN RULES (NDR) ASSESSMENT:</h3>

                <ul>
                    <xsl:variable name="chkNDRrules">
                        <xsl:call-template name="checkNDRrules">
                            <xsl:with-param name="rulesChk" select="//as:Rule"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="string-length($chkNDRrules) &gt; 292">
                            <xsl:variable name="resultsDetail"
                                select="distinct-values(tokenize($chkNDRrules,'\|'))"/>
                            <xsl:if test="contains($chkNDRrules,'anyAttribute:-')">
                                <li style="color:red">
                                    <strong>use of anyAttribute content model found on:</strong>
                                </li>
                                <xsl:for-each select="$resultsDetail">
                                    <xsl:if test="contains(.,'anyAttribute:-')">
                                        <li>- <xsl:value-of
                                                select="replace(substring-before(substring-after(.,'['),']'),'//','')"
                                            /></li>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:if>
                            <xsl:if test="contains($chkNDRrules,'any:-')">
                                <li style="color:red">
                                    <strong>use of any content model found on:</strong>
                                </li>
                                <xsl:for-each select="$resultsDetail">
                                    <xsl:if test="contains(.,'any:-')">
                                        <li>- <xsl:value-of
                                                select="replace(substring-before(substring-after(.,'['),']'),'//','')"
                                            /></li>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:if>
                            <xsl:if test="contains($chkNDRrules,'anySimpleType:-')">
                                <li style="color:red">
                                    <strong>use of anySimpleType content model found on:</strong>
                                </li>
                                <xsl:for-each select="$resultsDetail">
                                    <xsl:if test="contains(.,'anySimpleType:-')">
                                        <li>- <xsl:value-of
                                                select="replace(substring-before(substring-after(.,'['),']'),'//','')"
                                            /></li>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:if>
                            <xsl:if test="contains($chkNDRrules,'anyType:-')">
                                <li style="color:red">
                                    <strong>use of anyType content model found on:</strong>
                                </li>
                                <xsl:for-each select="$resultsDetail">
                                    <xsl:if test="contains(.,'anyType:-')">
                                        <li>- <xsl:value-of
                                                select="replace(substring-before(substring-after(.,'['),']'),'//','')"
                                            /></li>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:if>
                            <!-- choice is allowed in NDR when used in extension and exchange schemas. -->
                            <xsl:if test="contains($chkNDRrules,'choice:-')">
                                <li style="color:red">use of choice for singleton element node found
                                    on:</li>
                                <xsl:for-each select="$resultsDetail">
                                    <xsl:if test="contains(.,'choice:-')">
                                        <xsl:choose>
                                            <xsl:when test="contains(.,'/*[')">
                                                <li>- <xsl:value-of
                                                  select="replace(replace(replace(replace(substring-before(substring-after(.,'setChoice('),'])'),'//',''),'contains\(name\(\),',''),'\(name\(\)=',''),'\)','')"
                                                  />]</li>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <li>- <xsl:value-of
                                                  select="substring-before(substring-after(.,'setChoice('),')')"
                                                  /></li>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'attributenamerepr:-0'))">
                                <li style="color:red">attribute name does not match representation
                                    term rules: <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'attributenamerepr:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="attrNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badattributes:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$attrNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'elementnamerepr:-0'))">
                                <li style="color:red">element name does not match representation
                                    term rules: <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'elementnamerepr:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="elemNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badelements:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$elemNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'noAnnoAttributes:-0'))">
                                <li style="color:red">attributes missing annotation description:
                                        <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'noAnnoAttributes:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="attrAnnoNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badAnnoAttributes:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$attrAnnoNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'attributenames:-0'))">
                                <li style="color:red">attribute names not beginning with
                                        <xsl:value-of
                                        select="if($camelCaseMode='NIEM' or($camelCaseMode='LowerCase')) then 'lower case letter (a-z)' else 'upper case letter (A-Z)'"
                                    />: <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'attributenames:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="attrLowerNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badLowerAttributes:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$attrLowerNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'attributenamechar:-0'))">
                                <li style="color:red">attribute names contain underscore or period:
                                        <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'attributenamechar:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="attrCharNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badCharAttributes:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$attrCharNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'elements:-0'))">
                                <li style="color:red">elements missing annotation description:
                                        <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'elements:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="elemAnnoNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badAnnoElements:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$elemAnnoNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'elementnames:-0'))">
                                <li style="color:red">element names not beginning with <xsl:value-of
                                        select="if($camelCaseMode='LowerCase') then 'lower case letter (a-z)' else 'upper case letter (A-Z)'"
                                    />: <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'elementnames:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="elemNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badFirstelements:-'),'\'),'\|'))"/>
                                    <xsl:variable name="sortedElements">
                                        <xsl:perform-sort select="$elemNamesDetail">
                                            <xsl:sort/>
                                        </xsl:perform-sort>
                                    </xsl:variable>
                                    <xsl:for-each select="$sortedElements">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <xsl:if test="not(contains($chkNDRrules,'elementnamechar:-0'))">
                                <li style="color:red">element names contain underscore or period:
                                        <xsl:value-of
                                        select="xsd:integer(substring-before(substring-after($chkNDRrules,'elementnamechar:-'),'\')) -1"
                                    /></li>
                                <br/>
                                <font color="black">
                                    <xsl:variable name="elemCharNamesDetail"
                                        select="distinct-values(tokenize(substring-before(substring-after($chkNDRrules,'badCharElements:-'),'\'),'\|'))"/>
                                    <xsl:for-each select="$elemCharNamesDetail">
                                        <xsl:if test="string-length(.) &gt; 0"> - <xsl:value-of
                                                select="."/></xsl:if>
                                    </xsl:for-each>
                                </font>
                                <br/>
                            </xsl:if>
                            <p>
                                <font color="red">
                                    <strong>SCORE: <xsl:value-of
                                            select="cam2func:computeNDRscore($chkNDRrules)"
                                        /></strong>
                                </font>
                            </p>
                            <xsl:if test="contains($chkNDRrules,'namerepr:-')">
                                <xsl:choose>
                                    <xsl:when
                                        test="contains($chkNDRrules,'elementnamerepr:-0') and(contains($chkNDRrules,'attributenamerepr:-0'))"/>
                                    <xsl:otherwise>
                                        <p>
                                            <font color="black">
                                                <small>HINT: representation terms use indicates the
                                                  nature of the value carried by a leaf node in the
                                                  structure. The terms qualify the name of the node
                                                  and its content.<br/> Representation terms are
                                                  domain applicable and can be tailored accordingly.
                                                  The default list of terms provided for guidance
                                                  here are for a name that contains: 'Amount',
                                                  'Count', 'Total', 'BinaryObject', 'Graphic', 'Picture',
                                                  'Sound', 'Image', 'Video', 'Code', 'Category', 'Currency',
                                                  'EMail', 'DateTime', 'Date', 'Time', 'Indicator',
                                                  'Format', 'Length', 'Width', 'Height', 'Weight',
                                                  'Level', 'Measure', 'Mode', 'Method', 'Numeric',
                                                  'Number', 'Price', 'State', 'Status', 'Rank',
                                                  'Flag', 'Frequency', 'Size', 'Unit', 
                                                  'Value', 'Version', 'Rate', 'Required', 'Percent',
                                                  'Quantity', 'Qty', 'Description', 'Comment',
                                                  'Reason', 'Location', 'Instructions', 'Text',
                                                  'Title', 'Type', 'Year', 'Month', 'Day', 'Week', 'Name',
                                                  'URI', 'URL', or 'URN'; or that ends with
                                                  'Days', 'Hours', 'Minutes', 'ID', 'Id', XML, Votes, Score,
                                                  'Identification' or 'Identifier' <br/>A domain may
                                                  use specialized terms that are not in the overall
                                                  list shown here.</small>
                                            </font>
                                        </p>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:if>
                        </xsl:when>
                        <xsl:otherwise>
                            <li><font color="green">SCORE: 10.0</font> - no problems found</li>
                        </xsl:otherwise>
                    </xsl:choose>
                </ul>
                <p/>
                <h3>ISSUES AND WARNINGS:</h3>
                <xsl:if
                    test="not(concat($ignoreNillableIn,$ignoreSetLengthIn,$ignoreSetLimitIn)='falsefalsefalse')">
                    <ul>
                        <xsl:if test="not($ignoreNillableIn='false')">
                            <li>Ignore Nillable = <xsl:value-of select="$ignoreNillableIn"/></li>
                        </xsl:if>
                        <xsl:if test="not($ignoreSetLengthIn='false')">
                            <li>Ignore SetLength = <xsl:value-of select="$ignoreSetLengthIn"/></li>
                        </xsl:if>
                        <xsl:if test="not($ignoreSetLimitIn='false')">
                            <li>Ignore SetLimit = <xsl:value-of select="$ignoreSetLimitIn"/></li>
                        </xsl:if>
                        <!-- xsl:if test="not($ignoreAnyType='false')"><li>Ignore anyType = <xsl:value-of select="$ignoreAnyType"/></li></xsl:if -->
                    </ul>
                </xsl:if>
                <ul>
                    <xsl:variable name="warnings">
                        <xsl:if
                            test="//as:Rule/as:constraint[@action='excludeTree()'] or(//as:Rule/as:constraint[@action='excludeElement()'] ) or(//as:Rule/as:constraint[@action='excludeAttribute()']) ">
                            <li>
                                <b>Warning: Template contains
                                    excludeTree()/excludeElement()/excludeAttribute() rules.</b>
                                <ul>
                                    <xsl:variable name="excludes"
                                        select="//as:Rule/as:constraint[@action='excludeTree()']/../../@name"/>
                                    <xsl:variable name="excludeItems"
                                        select="distinct-values($excludes)"/>
                                    <xsl:for-each select="$excludeItems">
                                        <li>
                                            <xsl:value-of select="."/>
                                        </li>
                                    </xsl:for-each>
                                    <xsl:variable name="excludesE"
                                        select="//as:Rule/as:constraint[@action='excludeElement()']/../../@name"/>
                                    <xsl:variable name="excludeItemsE"
                                        select="distinct-values($excludesE)"/>
                                    <xsl:for-each select="$excludeItemsE">
                                        <li>
                                            <xsl:value-of select="."/>
                                        </li>
                                    </xsl:for-each>
                                    <xsl:if
                                        test="//as:Rule/as:constraint[@action='excludeAttribute()']">
                                        <xsl:variable name="excludesA"
                                            select="//as:Rule/as:constraint[@action='excludeAttribute()']/../../@name"/>
                                        <xsl:variable name="excludeItemsA"
                                            select="distinct-values($excludesA)"/>
                                        <xsl:for-each select="$excludeItemsA">
                                            <li> @ <xsl:value-of select="."/></li>
                                        </xsl:for-each>
                                    </xsl:if>
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:variable name="mixedContentCheck">
                            <xsl:apply-templates select="//as:Element" mode="mixedContent"/>
                        </xsl:variable>

                        <xsl:if test="$mixedContentCheck/li"><!-- Check for mixed content parent elements, with content restriction rules -->
                            <li>
                                <b>Use of mixed content model detected.</b>
                                <ul>
                                    <xsl:copy-of select="$mixedContentCheck"/>
                                </ul>
                            </li>
                        </xsl:if>

                        <xsl:if
                            test="$ignoreNillableIn='false' and //as:Rule/as:constraint[@action='makeNillable(xsd)']">
                            <li>
                                <b>Use of xsd nillable() can cause interoperability issues. Consider adding an attribute instead to convey meaning of empty items.</b>
                                <ul>
                                    <xsl:variable name="nills"
                                        select="//as:Rule/as:constraint[@action='makeNillable(xsd)']/../../@name"/>
                                    <xsl:variable name="nillItems" select="distinct-values($nills)"/>
                                    <xsl:for-each select="$nillItems">
                                        <xsl:variable name="name" select="."/>
                                        <xsl:variable name="nillChk" select="$rules//as:Element[@name = $name]/as:Rule/as:constraint[contains(@action,'makeNillable')][1]/@action"/>
                                        <xsl:variable name="optionalChk" select="$rules//as:Element[@name = $name]/as:Rule/as:constraint[contains(@action,'makeOptional')][1]/@action"/>
                                        <xsl:variable name="repeatChk" select="$rules//as:Element[@name = $name]/as:Rule/as:constraint[contains(@action,'makeRepeatable')][1]/@action"/>
                                        <li>
                                            <xsl:choose>
                                                <xsl:when test="string-length($optionalChk[1]) = 0 and(string-length($repeatChk[1]) = 0)"><font color='red'>ERROR: </font></xsl:when>
                                                <xsl:when test="string-length($optionalChk[1]) = 0 and(string-length($repeatChk[1]) &gt; 0)"><font color='red'>OCCURS ERROR: </font></xsl:when>
                                                <xsl:when test="string-length($optionalChk[1]) &gt; 0 and(string-length($repeatChk[1]) &gt; 0)"><font color='red'>DATA MODEL ERROR: </font></xsl:when>
                                                <xsl:otherwise>WARNING: </xsl:otherwise>
                                            </xsl:choose>
                                            <xsl:value-of select="$name"/> (<xsl:value-of select="substring-before(substring-after($nillChk[1],'('),')') "/>)
                                        </li>
                                    </xsl:for-each>
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:for-each select="//as:Rule[as:constraint/@action='orderChildren()']">
                            <xsl:if test="../as:Element/as:Rule/as:constraint[contains(@action,'setChoice')] ">
                                <li>
                                    <b>Rule orderChildren() will only successfully check the order
                                        of children where no choice nodes exist</b>
                                </li>
                            </xsl:if>
                        </xsl:for-each>
                        <xsl:if
                            test="$ignoreSetLimitIn='false' and //as:Rule/as:constraint[contains(@action,'setLimit(')]">
                            <li>
                                <b>Caution - use of setLimit() can cause interoperability
                                    issues.</b>
                                <ul>
                                    <xsl:variable name="limit"
                                        select="//as:Rule/as:constraint[contains(@action,'setLimit(')]/../../@name"/>
                                    <xsl:variable name="limitItems" select="distinct-values($limit)"/>
                                    <xsl:for-each select="$limitItems">
                                        <xsl:variable name="name" select="."/>
                                        <xsl:variable name="limitChk" select="$rules//as:Element[@name = $name]/as:Rule/as:constraint[contains(@action,'setLimit')]/@action"/>
                                        <li>
                                            <xsl:value-of select="$name"/> (<xsl:value-of select="substring-before(substring-after($limitChk[1],'('),')') "/>)
                                        </li>
                                    </xsl:for-each>
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="$ignoreNameRuleType='false' and //as:Rule/as:constraint[contains(@action,'setLength(')]">
                            <li>
                                <b>Caution - string item has length restriction - may be interoperability issue</b>
                                <ul>
                                    <xsl:variable name="length" select="//as:Rule/as:constraint[contains(@action,'setLength(')]/../../@name"/>
                                    <xsl:variable name="lengthItems" select="distinct-values($length)"/>
                                    <xsl:for-each select="$lengthItems">
                                        <xsl:variable name="this"/>
                                        <xsl:variable name="name" select="upper-case(.)"/>
                                        <xsl:variable name="chkItem" select="cam2func:nameTypeCorrelation(if(contains($name,':')) then substring-after($name,':') else $name,.)"/>
                                        <xsl:if test="string-length($chkItem) &gt; 0">
                                            <xsl:variable name="lengthChk" select="$rules//as:Element[@name = $chkItem]/as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                            <xsl:variable name="lengthChkAttr" select="$rules//as:Attribute[@name = $chkItem]/as:Rule/as:constraint[contains(@action,'setLength')]/@action"/>
                                            <xsl:variable name="lengthDetail" select="if(string-length($lengthChk[1]) &gt; 0) then substring-before(substring-after($lengthChk[1],'('),')') else substring-before(substring-after($lengthChkAttr[1],'('),')') "/>
                                            <xsl:variable name="first-number" select="if(contains($lengthDetail,'-')) then xsd:integer(substring-before($lengthDetail,'-')) else 1"/>
                                            <xsl:variable name="second-number" select="if(contains($lengthDetail,'-')) then xsd:integer(substring-after($lengthDetail,'-')) else xsd:integer($lengthDetail)"/>
                                            <xsl:if test="not($first-number = $second-number) or(not(contains($lengthDetail,'-')))">                                            
                                                <li>
                                                  <xsl:value-of select="$chkItem"/> (<xsl:value-of select="$lengthDetail"/>)
                                               </li>
                                            </xsl:if>
                                        </xsl:if>
                                        <!-- xsl:choose>
                                            <xsl:when test="ends-with($name,'CODE')"/>
                                            <xsl:when test="ends-with($name,'CD')"/>
                                            <xsl:when test="ends-with($name,'ID')"/>
                                            <xsl:when test="ends-with($name,'TYPE')"/>
                                            <xsl:when test="contains($name,'IND')"/>
                                            <xsl:when test="contains($name,'IDENTIFIER')"/>
                                            <xsl:when test="contains($name,'NUMBER')"/>
                                            <xsl:when test="contains($name,'NUM')"/>
                                            <xsl:when test="contains($name,'DATE')"/>
                                            <xsl:when test="contains($name,'TIME')"/>
                                            <xsl:when test="ends-with($name,'FLAG')"/>
                                            <xsl:when test="contains($name,'AMOUNT')"/>
                                            <xsl:when test="contains($name,'VALUE')"/>
                                            <xsl:otherwise><li><xsl:value-of select="."/></li></xsl:otherwise>
                                        </xsl:choose -->
                                    </xsl:for-each>
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="$ignoreNameRuleType='false'">
                            <xsl:variable name="attribute-xpaths">
                                <xsl:for-each select="//as:Attribute">
                                    <xsl:element name="item" exclude-result-prefixes="#all">
                                        <xsl:attribute name="name" select="@name"/>
                                        <xsl:attribute name="xpath" select="if(as:Rule/as:constraint[1]/@xpath) then as:Rule/as:constraint[1]/@xpath else ''"/>
                                    </xsl:element>
                                </xsl:for-each>
                            </xsl:variable>
                            <!-- xsl:variable name="element-xpaths">
                                <xsl:for-each select="//as:Element">
                                    <xsl:element name="item" exclude-result-prefixes="#all">
                                        <xsl:attribute name="name" select="@name"/>
                                        <xsl:attribute name="xpath" select="if(as:Rule/as:constraint[1]/@xpath) then as:Rule/as:constraint[1]/@xpath else '%'"/>
                                    </xsl:element>
                                </xsl:for-each>
                            </xsl:variable -->
                            
                            <!-- xsl:variable name="date-time-items-attr-xpath">
                                <xsl:for-each select="//as:Attribute/upper-case(@name)">
                                    <xsl:choose>
                                        <xsl:when test="contains(replace(replace(replace(replace(.,'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(.,'TIME'))">
                                            <xsl:variable name="this" select="position()"/>
                                            <xsl:value-of select="$attribute-xpaths//item[$this]/@xpath"/>
                                        </xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:variable -->
                            <xsl:variable name="date-time-items-attr-rules">
                                <xsl:for-each select="//as:Attribute">
                                    <!-- xsl:message> - <xsl:value-of select="@name"/></xsl:message -->
                                    <xsl:choose>
                                        <xsl:when test="contains(replace(replace(replace(replace(upper-case(@name),'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(upper-case(@name),'TIME'))">
                                            <xsl:element name="item">
                                                <xsl:attribute name="name" select="@name"/>
                                                <xsl:for-each select="as:Rule//as:constraint">
                                                    <xsl:choose>
                                                        <xsl:when test="contains(./@action,'setDateMask')">
                                                            <xsl:attribute name="xpath"><xsl:value-of select="./@xpath"/></xsl:attribute>
                                                        </xsl:when>
                                                        <xsl:when test="contains(./@action,'datatype') and(contains(./@action,'date'))">
                                                            <xsl:attribute name="xpath"><xsl:value-of select="./@xpath"/></xsl:attribute>
                                                        </xsl:when>
                                                        <xsl:otherwise/>
                                                    </xsl:choose>
                                                </xsl:for-each>
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:variable>
                            
                            <xsl:variable name="date-time-items-elem-rules">
                                <xsl:for-each select="//as:Element">
                                    <!-- xsl:message> - <xsl:value-of select="@name"/></xsl:message -->
                                    <xsl:choose>
                                        <xsl:when test="contains(replace(replace(replace(replace(upper-case(@name),'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(upper-case(@name),'TIME'))">
                                            <xsl:element name="item">
                                                <xsl:attribute name="name" select="@name"/>
                                            <xsl:for-each select="as:Rule//as:constraint">
                                                <xsl:choose>
                                                    <xsl:when test="contains(./@action,'setDateMask')">
                                                        <xsl:attribute name="xpath"><xsl:value-of select="./@xpath"/></xsl:attribute>
                                                    </xsl:when>
                                                    <xsl:when test="contains(./@action,'datatype') and(contains(./@action,'date'))">
                                                        <xsl:attribute name="xpath"><xsl:value-of select="./@xpath"/></xsl:attribute>
                                                    </xsl:when>
                                                   <xsl:otherwise/>
                                                </xsl:choose>
                                            </xsl:for-each>
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:variable>
                            
                            <!-- xsl:variable name="date-time-items-attr-xpath"
                                select="//as:Attribute[contains(replace(replace(replace(replace(upper-case(@name),'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(upper-case(@name),'TIME'))]/as:Rule/as:constraint/@xpath[1]"/>
                            <xsl:variable name="date-time-items-elem-xpath"
                                select="//as:Element[contains(replace(replace(replace(replace(upper-case(@name),'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(upper-case(@name),'TIME'))]/as:Rule/as:constraint/@xpath[1]"/ -->
                            <!-- xsl:variable name="test-date-type-items-attr"
                                select="//as:Attribute[contains(replace(replace(replace(replace(upper-case(@name),'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(upper-case(@name),'TIME'))]/as:Rule//as:constraint"/ -->
                            <!-- xsl:variable name="test-date-type-items-elem"
                                select="//as:Element[contains(replace(replace(replace(replace(upper-case(@name),'DATEXT','WW'),'LIDATE','XX'),'UPDATE','YY'),'DIDATE','ZZZ'),'DATE') or (ends-with(upper-case(@name),'TIME'))]/as:Rule//as:constraint"/ -->
                            <!-- xsl:variable name="found-attr-mask">
                                <xsl:if test="count($test-date-type-items-attr) &gt; 0">
                                    <xsl:for-each select="$test-date-type-items-attr"><xsl:if test="contains(./@action,'setDateMask')"><xsl:value-of select="./@xpath"/>|</xsl:if></xsl:for-each>
                                </xsl:if>
                            </xsl:variable -->
                            <!-- xsl:variable name="found-elem-mask">
                                <xsl:if test="count($test-date-type-items-elem) &gt; 0">
                                    <xsl:for-each select="$test-date-type-items-elem"><xsl:if test="contains(./@action,'setDateMask')"><xsl:value-of select="./@xpath"/>|</xsl:if></xsl:for-each>
                                </xsl:if>
                            </xsl:variable -->
                            <!-- xsl:message> - <xsl:copy-of select="$date-time-items-elem-rules"/> [<xsl:value-of select="$found-elem-mask"></xsl:value-of>]</xsl:message -->
                            <xsl:variable name="attr-missing-mask">
                                <xsl:for-each select="distinct-values($date-time-items-attr-rules//item/@name)">
                                    <xsl:variable name="this" select="."/>
                                    <xsl:if test="$date-time-items-attr-rules/item[@name = $this][not(@xpath)]"><xsl:value-of select="$this"/>|</xsl:if>
                                </xsl:for-each>
                                <!-- xsl:for-each select="distinct-values($date-time-items-attr-xpath)"><xsl:value-of select="cam2func:maskItemsCorrelation(.,$found-attr-mask)"/></xsl:for-each -->
                                    <!-- xsl:choose>
                                        <xsl:when test="contains(.,'*')"/>
                                        <xsl:when test="ends-with(upper-case(.),'DATES')"/>
                                        <xsl:when test="ends-with(upper-case(.),'TION')"/>
                                        <xsl:when test="ends-with(upper-case(.),'DESC')"/>
                                        <xsl:when test="ends-with(upper-case(.),'ID')"/>
                                        <xsl:when test="contains($found-attr-mask,replace(.,'//','/'))"/>
                                        <xsl:otherwise><xsl:value-of select="."/>|</xsl:otherwise>
                                    </xsl:choose -->
                            </xsl:variable>
                            <xsl:variable name="elem-missing-mask">
                                <xsl:for-each select="distinct-values($date-time-items-elem-rules//item/@name)">
                                    <xsl:variable name="this" select="."/>
                                    <xsl:if test="$date-time-items-elem-rules/item[@name = $this][not(@xpath)]"><xsl:value-of select="$this"/>|</xsl:if>
                                    <!-- xsl:value-of select="cam2func:maskItemsCorrelation(.,$found-elem-mask)"/ -->
                                </xsl:for-each>
                                    <!-- xsl:choose>
                                        <xsl:when test="contains(.,'*')"/>
                                        <xsl:when test="ends-with(upper-case(.),'DATES')"/>
                                        <xsl:when test="ends-with(upper-case(.),'TION')"/>
                                        <xsl:when test="ends-with(upper-case(.),'DESC')"/>
                                        <xsl:when test="ends-with(upper-case(.),'ID')"/>
                                        <xsl:when test="contains($found-elem-mask,replace(.,'//','/'))"/>
                                        <xsl:otherwise><xsl:value-of select="."/>|</xsl:otherwise>
                                    </xsl:choose -->
                            </xsl:variable>
                            <xsl:variable name="element-missing-dates-chk">
                                <xsl:for-each select="tokenize($elem-missing-mask,'\|')">
                                    <xsl:variable name="Parent" select="replace(.,'//','/')"/>
                                    <xsl:call-template name="checkForChildDate">
                                        <xsl:with-param name="Parent" select="$Parent"/>
                                    </xsl:call-template>
                                </xsl:for-each>
                            </xsl:variable>

                            <xsl:if test="string-length($attr-missing-mask) &gt; 0 or(string-length($elem-missing-mask) &gt; 0 )">
                                <li>
                                    <b>Warning - date or time items with no content mask</b>
                                    <ul>
                                        <xsl:variable name="attr-items"
                                            select="tokenize($attr-missing-mask,'\|')"/>
                                        <xsl:for-each select="$attr-items"><xsl:if test="string-length(.) &gt; 0"><li><xsl:value-of select="replace(.,'//','')"/></li></xsl:if></xsl:for-each>
                                        <xsl:variable name="elem-item-details" select="tokenize($element-missing-dates-chk,'Parent: ')"/>
                                        <xsl:for-each select="$elem-item-details">
                                            <xsl:variable name="elem-details" select="tokenize(.,'\|')"/>
                                            <xsl:if test="count($elem-details) &lt; 3">
                                                <xsl:for-each select="$elem-details">
                                                  <xsl:if test="string-length(.) &gt; 0"><li><xsl:value-of select="replace(.,'//','')"/></li></xsl:if>
                                                </xsl:for-each>
                                            </xsl:if>
                                        </xsl:for-each>
                                    </ul>
                                </li>
                            </xsl:if>
                        </xsl:if>
                        <xsl:if test="$ignoreNameRuleType='false'">
                            <xsl:variable name="code-type-items-attr"
                                select="//as:Attribute[(contains(upper-case(@name),'CODE') and(not(contains(@name,'ZIP')))) or (ends-with(upper-case(@name),'TYPE')) or (ends-with(upper-case(@name),'FLAG')) or (ends-with(upper-case(@name),'CD')) or (ends-with(upper-case(@name),'INDICATOR'))]/as:Rule/as:constraint/@xpath[1]"/>
                            <xsl:variable name="code-type-items-elem"
                                select="//as:Element[(contains(upper-case(@name),'CODE') and(not(contains(@name,'ZIP')))) or (ends-with(upper-case(@name),'TYPE')) or (ends-with(upper-case(@name),'FLAG')) or (ends-with(upper-case(@name),'CD')) or (ends-with(upper-case(@name),'INDICATOR'))]/as:Rule/as:constraint/@xpath[1]"/>
                            <xsl:variable name="test-code-type-items-attr"
                                select="//as:Attribute[(contains(upper-case(@name),'CODE') and(not(contains(@name,'ZIP')))) or (ends-with(upper-case(@name),'TYPE')) or (ends-with(upper-case(@name),'FLAG')) or (ends-with(upper-case(@name),'CD')) or (ends-with(upper-case(@name),'INDICATOR'))]/as:Rule//as:constraint"/>
                            <xsl:variable name="test-code-type-items-elem"
                                select="//as:Element[(contains(upper-case(@name),'CODE') and(not(contains(@name,'ZIP')))) or (ends-with(upper-case(@name),'TYPE')) or (ends-with(upper-case(@name),'FLAG')) or (ends-with(upper-case(@name),'CD')) or (ends-with(upper-case(@name),'INDICATOR'))]/as:Rule//as:constraint"/>
                            <!-- xsl:variable name="test-URL-type-items-attr" select="//as:Attribute[contains(upper-case(@name),'URL') or (contains(upper-case(@name),'URI')) or (contains(upper-case(@name),'HTTP'))]/as:Rule//as:constraint"/>
                            <xsl:variable name="test-URL-type-items-elem" select="//as:Element[contains(upper-case(@name),'URL') or (contains(upper-case(@name),'URI')) or (contains(upper-case(@name),'HTTP'))]/as:Rule//as:constraint"/ -->
                            <xsl:variable name="found-attr-restricted">
                                <xsl:if test="count($test-code-type-items-attr) &gt; 0">
                                    <xsl:for-each select="$test-code-type-items-attr">
                                        <xsl:if
                                            test="contains(./@action,'restrictValues') or contains(./@action,'lookup') or contains(./@action,'Mask') or contains(./@action,'boolean')"
                                                ><xsl:value-of select="./@xpath"/>|</xsl:if>
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:variable>
                            <xsl:variable name="found-elem-restricted">
                                <xsl:if test="count($test-code-type-items-elem) &gt; 0">
                                    <xsl:for-each select="$test-code-type-items-elem">
                                        <xsl:if
                                            test="contains(./@action,'restrictValues') or contains(./@action,'lookup') or contains(./@action,'Mask') or contains(./@action,'boolean')"
                                                ><xsl:value-of select="./@xpath"/>|</xsl:if>
                                    </xsl:for-each>
                                </xsl:if>
                            </xsl:variable>
                            <xsl:variable name="attr-missing-values">
                                <xsl:for-each select="distinct-values($code-type-items-attr)">
                                    <xsl:value-of
                                        select="cam2func:codeItemsCorrelation(.,$found-attr-restricted)"/>
                                    <!-- xsl:choose>
                                        <xsl:when test="contains(.,'*')"/>
                                        <xsl:when test="ends-with(upper-case(.),'TION')"/>
                                        <xsl:when test="ends-with(upper-case(.),'DESC')"/>
                                        <xsl:when test="ends-with(upper-case(.),'ID')"/>
                                        <xsl:when test="ends-with(upper-case(.),'URN')"/>
                                        <xsl:when test="contains($found-attr-restricted,replace(.,'//','/'))"/>
                                        <xsl:otherwise><xsl:value-of select="."/>|</xsl:otherwise>
                                    </xsl:choose -->
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:variable name="elem-missing-values">
                                <xsl:for-each select="distinct-values($code-type-items-elem)">
                                    <xsl:value-of
                                        select="cam2func:codeItemsCorrelation(.,$found-elem-restricted)"/>
                                    <!-- xsl:choose>
                                        <xsl:when test="contains(.,'*')"/>
                                        <xsl:when test="ends-with(upper-case(.),'TION')"/>
                                        <xsl:when test="ends-with(upper-case(.),'DESC')"/>
                                        <xsl:when test="ends-with(upper-case(.),'ID')"/>
                                        <xsl:when test="ends-with(upper-case(.),'URN')"/>
                                        <xsl:when test="contains($found-elem-restricted,replace(.,'//','/'))"/>
                                        <xsl:otherwise><xsl:value-of select="."/>|</xsl:otherwise>
                                    </xsl:choose -->
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:variable name="element-missing-values-chk">
                                <xsl:for-each select="tokenize($elem-missing-values,'\|')">
                                    <xsl:variable name="Parent" select="replace(.,'//','/')"/>
                                    <xsl:call-template name="checkForChildCode">
                                        <xsl:with-param name="Parent" select="$Parent"/>
                                    </xsl:call-template>
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:if
                                test="string-length($attr-missing-values) &gt; 0 or(string-length($elem-missing-values) &gt; 0 )">
                                <li>
                                    <b>Warning - code, type, indicator or flag items with no allowed values restriction</b>
                                    <ul>
                                        <xsl:variable name="attr-items"
                                            select="tokenize($attr-missing-values,'\|')"/>
                                        <xsl:for-each select="$attr-items">
                                            <xsl:variable name="NameChk"
                                                select="upper-case(substring-after(replace(.,'//',''),'/'))"/>
                                            <xsl:if
                                                test="string-length(.) &gt; 0 and(not(ends-with($NameChk,'TEXT')) or(ends-with($NameChk,'ID')))">
                                                <li>
                                                  <xsl:value-of select="replace(.,'//','')"/>
                                                </li>
                                            </xsl:if>
                                        </xsl:for-each>
                                        <xsl:variable name="elem-item-details"
                                            select="tokenize($element-missing-values-chk,'Parent: ')"/>
                                        <xsl:for-each select="$elem-item-details">
                                            <xsl:variable name="elem-details"
                                                select="tokenize(.,'\|')"/>
                                            <xsl:for-each select="$elem-details">
                                                <xsl:variable name="NameChk"
                                                  select="upper-case(substring-after(replace(.,'//',''),'/'))"/>
                                                <xsl:if
                                                  test="string-length(.) &gt; 0 and(not(ends-with($NameChk,'TEXT')) or(ends-with($NameChk,'ID')))">
                                                  <li>
                                                  <xsl:value-of select="replace(.,'//','')"/>
                                                  </li>
                                                </xsl:if>
                                            </xsl:for-each>
                                        </xsl:for-each>
                                    </ul>
                                </li>
                            </xsl:if>
                        </xsl:if>
                        <xsl:if
                            test="$ignoreFixedValues='false' and //as:Attribute[not(starts-with(@exampleValue,'%') and ends-with(normalize-space(@exampleValue),'%'))]">
                            <li>
                                <b>Caution - use of fixed value attributes can cause
                                    interoperability issues.</b>
                                <ul>
                                    <xsl:variable name="setFixed"
                                        select="//as:Attribute[not(starts-with(@exampleValue,'%') and ends-with(normalize-space(@exampleValue),'%'))]/@name"/>
                                    <xsl:variable name="setFixedItems"
                                        select="distinct-values($setFixed)"/>
                                    <xsl:for-each select="$setFixedItems">
                                        <xsl:variable name="name" select="."/>
                                        <li>
                                            <xsl:value-of select="."/>
                                        </li>
                                    </xsl:for-each>
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if
                            test="$ignoreFixedValues='false' and //as:Element[string-length(@exampleValue) &gt; 0 and(not(starts-with(@exampleValue,'%') and ends-with(normalize-space(@exampleValue),'%')))]">
                            <li>
                                <b>Caution - use of fixed value elements can cause interoperability
                                    issues.</b>
                                <ul>
                                    <xsl:variable name="setFixed"
                                        select="//as:Element[string-length(@exampleValue) &gt; 0 and(not(starts-with(@exampleValue,'%') and ends-with(normalize-space(@exampleValue),'%')))]/@name"/>
                                    <!-- xsl:variable name="setValues" select="//as:Element[string-length(@exampleValue) &gt; 0 and(not(starts-with(@exampleValue,'%') and ends-with(normalize-space(@exampleValue),'%')))]/concat(@name,'-',@exampleValue,'|')"/ -->
                                    <xsl:variable name="setFixedItems"
                                        select="distinct-values($setFixed)"/>
                                    <xsl:for-each select="$setFixedItems">
                                        <xsl:variable name="name" select="."/>
                                        <li>
                                            <xsl:value-of select="."/>
                                        </li>
                                    </xsl:for-each>
                                </ul>
                            </li>
                        </xsl:if>
                        <xsl:if test="$ignoreNS='false'">
                            <xsl:variable name="chkNamespaces" select="//as:Namespaces"/>
                            <xsl:variable name="chkNSdetails">
                                <xsl:for-each select="$chkNamespaces/as:namespace">
                                    <xsl:variable name="chkNS" select="@prefix"/>
                                    <xsl:variable name="chkURN" select="node()"/>
                                    <xsl:if
                                        test="count($chkNamespaces/as:namespace[@prefix=$chkNS]) &gt; 1 or(count($chkNamespaces/as:namespace[node()=$chkURN]) &gt; 1)">
                                        <thisNS prefix="{@prefix}" url="{text()}"/>
                                    </xsl:if>
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:if test="count($chkNSdetails/*) &gt; 0">
                                <li>
                                    <b>Caution - duplicate use of namespace prefix or URN detected,
                                        can cause schema structure interpretation issues.</b>
                                    <ul>
                                        <xsl:for-each select="$chkNSdetails/thisNS">
                                            <li>prefix: <xsl:value-of select="@prefix"/> -> urn:
                                                  <xsl:value-of select="@url"/></li>
                                        </xsl:for-each>
                                    </ul>
                                </li>
                            </xsl:if>
                        </xsl:if>
                        <!-- Check for numeric items without content mask -->
                        <xsl:variable name="numeric-items-attr"
                            select="//as:Attribute[not((contains(upper-case(@name),'COUNTERMEASURE'))) and(contains(upper-case(@name),'NUMBER') or(contains(upper-case(@name),'NUMERIC')) or(contains(upper-case(@name),'QUANTITY')) or(contains(upper-case(@name),'VALUE')) or(contains(upper-case(@name),'PRICE')) or(contains(upper-case(@name),'AMOUNT')) or(contains(upper-case(@name),'PERCENT')) or(ends-with(upper-case(@name),'COUNT')) or(contains(upper-case(@name),'LENGTH')) or(contains(upper-case(@name),'WEIGHT')) or(contains(upper-case(@name),'HEIGHT')) or(contains(upper-case(@name),'FREQUENCY')) or(contains(upper-case(@name),'MEASURE')) or(ends-with(upper-case(@name),'RATE'))  or(ends-with(upper-case(@name),'QTY')) or (ends-with(upper-case(@name),'NUM')) or(ends-with(upper-case(@name),'SIZE')))]/as:Rule//as:constraint/@xpath[1]"/>
                        <xsl:variable name="numeric-items-elem"
                            select="//as:Element[not((contains(upper-case(@name),'COUNTERMEASURE'))) and(contains(upper-case(@name),'NUMBER') or(contains(upper-case(@name),'NUMERIC')) or(contains(upper-case(@name),'QUANTITY'))or(contains(upper-case(@name),'VALUE')) or(contains(upper-case(@name),'PRICE')) or(contains(upper-case(@name),'AMOUNT')) or(contains(upper-case(@name),'PERCENT')) or(ends-with(upper-case(@name),'COUNT')) or(contains(upper-case(@name),'LENGTH')) or(contains(upper-case(@name),'WEIGHT')) or(contains(upper-case(@name),'HEIGHT')) or(contains(upper-case(@name),'FREQUENCY')) or(contains(upper-case(@name),'MEASURE')) or(ends-with(upper-case(@name),'RATE')) or(ends-with(upper-case(@name),'QTY')) or (ends-with(upper-case(@name),'NUM')) or(ends-with(upper-case(@name),'SIZE')))]/as:Rule//as:constraint/@xpath[1]"/>
                        <xsl:variable name="test-numeric-items-attr"
                            select="//as:Attribute[not((contains(upper-case(@name),'COUNTERMEASURE'))) and(contains(upper-case(@name),'NUMBER') or(contains(upper-case(@name),'NUMERIC')) or(contains(upper-case(@name),'QUANTITY')) or(contains(upper-case(@name),'VALUE')) or(contains(upper-case(@name),'PRICE')) or(contains(upper-case(@name),'AMOUNT')) or(contains(upper-case(@name),'PERCENT')) or(ends-with(upper-case(@name),'COUNT')) or(contains(upper-case(@name),'LENGTH')) or(contains(upper-case(@name),'WEIGHT')) or(contains(upper-case(@name),'HEIGHT')) or(contains(upper-case(@name),'FREQUENCY')) or(contains(upper-case(@name),'MEASURE')) or(ends-with(upper-case(@name),'RATE'))  or(ends-with(upper-case(@name),'QTY')) or (ends-with(upper-case(@name),'NUM')) or(ends-with(upper-case(@name),'SIZE')))]/as:Rule//as:constraint"/>
                        <xsl:variable name="test-numeric-items-elem"
                            select="//as:Element[not((contains(upper-case(@name),'COUNTERMEASURE'))) and(contains(upper-case(@name),'NUMBER') or(contains(upper-case(@name),'NUMERIC')) or(contains(upper-case(@name),'QUANTITY'))or(contains(upper-case(@name),'VALUE')) or(contains(upper-case(@name),'PRICE')) or(contains(upper-case(@name),'AMOUNT')) or(contains(upper-case(@name),'PERCENT')) or(ends-with(upper-case(@name),'COUNT')) or(contains(upper-case(@name),'LENGTH')) or(contains(upper-case(@name),'WEIGHT')) or(contains(upper-case(@name),'HEIGHT')) or(contains(upper-case(@name),'FREQUENCY')) or(contains(upper-case(@name),'MEASURE')) or(ends-with(upper-case(@name),'RATE')) or(ends-with(upper-case(@name),'QTY')) or (ends-with(upper-case(@name),'NUM')) or(ends-with(upper-case(@name),'SIZE')))]/as:Rule//as:constraint"/>

                        <xsl:variable name="found-attr-num-mask">
                            <xsl:if test="count($test-numeric-items-attr) &gt; 0">
                                <xsl:for-each select="$test-numeric-items-attr">
                                    <xsl:choose>
                                        <xsl:when test="contains(./@action,'setNumberMask')"><xsl:value-of select="./@xpath"/>|</xsl:when>
                                        <xsl:when test="contains(./@action,'datatype')">
                                            <xsl:variable name="typeName" select="substring-after(substring-before(./@action,')'),'(')"/>
                                            <xsl:choose>
                                                <xsl:when test="$typeName = 'decimal'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="$typeName = 'float'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="$typeName = 'integer'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="contains($typeName,'Integer')"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="$typeName = 'int'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:otherwise/>
                                            </xsl:choose>
                                        </xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:variable>
                        <xsl:variable name="found-elem-num-mask">
                            <xsl:if test="count($test-numeric-items-elem) &gt; 0">
                                <xsl:for-each select="$test-numeric-items-elem">
                                    <xsl:choose>
                                        <xsl:when test="contains(./@action,'setNumberMask')"><xsl:value-of select="./@xpath"/>|</xsl:when>
                                        <xsl:when test="contains(./@action,'datatype')">
                                            <xsl:variable name="typeName" select="substring-after(substring-before(./@action,')'),'(')"/>
                                            <xsl:choose>
                                                <xsl:when test="$typeName = 'decimal'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="$typeName = 'float'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="$typeName = 'integer'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="contains($typeName,'Integer')"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:when test="$typeName = 'int'"> <xsl:value-of select="./@xpath"/>|</xsl:when>
                                                <xsl:otherwise/>
                                            </xsl:choose>
                                        </xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>                                  
                                </xsl:for-each>
                            </xsl:if>
                        </xsl:variable>
                        <xsl:variable name="attr-missing-num-mask">
                            <xsl:for-each select="distinct-values($numeric-items-attr)">
                                <xsl:value-of
                                    select="cam2func:numericRuleMatch(.,$found-attr-num-mask)"/>
                                <!-- Check for numeric types - there's a lot of them -->
                            </xsl:for-each>
                        </xsl:variable>
                        <xsl:variable name="elem-missing-num-mask">
                            <xsl:for-each select="distinct-values($numeric-items-elem)">
                                <xsl:variable name="this" select="."/>
                                <xsl:value-of
                                    select="cam2func:numericRuleMatch(.,$found-elem-num-mask)"/>
                                <!-- Check for numeric types - there's a lot of them -->
                            </xsl:for-each>
                        </xsl:variable>

                        <xsl:if
                            test="string-length($attr-missing-num-mask) &gt; 0 or(string-length($elem-missing-num-mask) &gt; 0 )">
                            <li>
                                <b>Warning - numeric items with no content mask</b>
                                <ul>
                                    <xsl:variable name="attr-num-items"
                                        select="tokenize($attr-missing-num-mask,'\|')"/>
                                    <xsl:for-each select="distinct-values($attr-num-items)">
                                        <xsl:if test="string-length(.) &gt; 0">
                                            <xsl:variable name="NameChk"
                                                select="upper-case(substring-after(replace(.,'//',''),'/'))"/>
                                            <xsl:if
                                                test="not(contains($NameChk,'PHONENUM') or(contains($NameChk,'VALUETEXT')) or(ends-with($NameChk,'TEXT')) or(ends-with($NameChk,'ID')))">
                                                <li>
                                                  <xsl:value-of select="replace(.,'//','')"/>
                                                </li>
                                            </xsl:if>
                                        </xsl:if>
                                    </xsl:for-each>
                                    <xsl:variable name="elem-num-details"
                                        select="tokenize($elem-missing-num-mask,'\|')"/>
                                    <xsl:for-each select="distinct-values($elem-num-details)">
                                        <xsl:if test="string-length(.) &gt; 0">
                                            <xsl:variable name="NameChk"
                                                select="upper-case(substring-after(replace(.,'//',''),'/'))"/>
                                            <xsl:if
                                                test="not(contains($NameChk,'PHONENUM') or(contains($NameChk,'VALUETEXT')) or(ends-with($NameChk,'TEXT')) or(ends-with($NameChk,'ID')))">
                                                <li>
                                                  <xsl:value-of select="replace(.,'//','')"/>
                                                </li>
                                            </xsl:if>
                                        </xsl:if>
                                    </xsl:for-each>
                                </ul>
                            </li>
                        </xsl:if>

                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="count($warnings/li) &gt; 0">
                            <xsl:copy-of select="$warnings"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <li>No problems found</li>
                        </xsl:otherwise>
                    </xsl:choose>
                </ul>
                <p/>
                <h3>EXTERNALS:</h3>
                <table>
                    <tr>
                        <td>
                            <xsl:value-of select="$nbsp"/>
                        </td>
                        <td>
                            <b>Namespace</b>
                        </td>
                        <td>
                            <b>URL</b>
                        </td>
                    </tr>
                    <xsl:for-each select="//as:Namespaces/as:namespace">
                        <tr>
                            <td>
                                <xsl:value-of select="$nbsp"/>
                            </td>
                            <td>
                                <xsl:value-of select="@prefix"/>
                            </td>
                            <td>
                                <xsl:value-of select="."/>
                            </td>
                        </tr>
                    </xsl:for-each>
                </table>
                <xsl:if test="$lookupTables//dblook:DbLookupList | look:LookupList">
                    <h4>LOOKUP CODELIST TABLES</h4>
                    <!-- xsl:message><xsl:copy-of select="$lookupTables"/> - <xsl:value-of select="/as:CAM/as:AssemblyStructure//as:constraint[starts-with(@action,'lookup(')]/@action"/> </xsl:message -->
                    <xsl:variable name="lookupsUsed">
                        <xsl:for-each
                            select="/as:CAM/as:AssemblyStructure//as:constraint[starts-with(@action,'lookup(')]">
                            <xsl:variable name="chkLookup" select="substring-before(substring-after(@action,'lookup('),')')"/> 
                            <xsl:choose>
                                <xsl:when test="$lookupTables//dblook:DbLookupList[@name = $chkLookup]"/><!-- Matched a SQL lookup -->
                                <xsl:otherwise>
                                    <xsl:if
                                        test="not($lookupTables//look:LookupList[@name = $chkLookup]/look:item/@name)">
                                        <xsl:element name="badLookup">
                                            <xsl:attribute name="name" select="$chkLookup"/>
                                            <xsl:attribute name="xpath" select="@xpath"/>
                                        </xsl:element>
                                    </xsl:if>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:variable>
                        
                    <xsl:variable name="badLookups" select="distinct-values($lookupsUsed//@name)"/>
                    <xsl:if test="$lookupsUsed//@name">
                        <h4>
                            <font color="red">MISSING LOOKUP TABLES</font>
                        </h4>
                        <table>
                            <tr>
                                <th/>
                                <th>Table</th>
                                <th>References</th>
                            </tr>
                            <xsl:for-each select="$badLookups">
                                <xsl:variable name="this" select="."/>
                                <xsl:variable name="badXPaths"
                                    select="distinct-values($lookupsUsed//badLookup[@name=$this]/@xpath)"/>
                                <tr>
                                    <td> </td>
                                    <td><font color="red"><xsl:value-of select="$this"/></font></td>
                                    <td><xsl:value-of select="$badXPaths"/></td>
                                </tr>
                            </xsl:for-each>
                        </table>
                        <h4>REFERENCED LOOKUP TABLES</h4>
                    </xsl:if>
                </xsl:if>
                <table>
                    <xsl:for-each select="$lookupTables//dblook:DbLookupList">
                        <tr>
                            <td> </td>
                            <td>
                                <xsl:value-of select="@name"/> -> <xsl:value-of select="."/>
                            </td>
                        </tr>
                    </xsl:for-each>
                    <xsl:for-each select="$lookupTables//look:LookupList/@name">
                        <tr>
                            <td> </td>
                            <td>
                                <xsl:value-of select="."/>
                            </td>
                        </tr>
                    </xsl:for-each>
                </table>
                <p/>
                <h3>SPELLING REVIEW:</h3>
                <p> Cut and paste the names list below into a word processor to perform spelling
                    check on element and attribute names. <br/>
                    <small>x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x</small></p>
                <table>
                    <tr>
                        <td>
                            <xsl:value-of select="$nbsp"/>
                        </td>
                        <td>
                            <b>Element names</b>
                        </td>
                    </tr>
                    <xsl:for-each select="distinct-values($allElements/allElements//element/@name)">
                        <xsl:variable name="newname" select="."/>
                        <xsl:variable name="newnameValue"
                            select="if(contains($newname,':')) then substring-after($newname,':') else $newname"/>
                        <xsl:variable name="splitNewName">
                            <xsl:value-of
                                select="camed:splitNewWords($newnameValue,$newnameValue,'true')"/>
                        </xsl:variable>
                        <xsl:variable name="spellChkName">
                            <xsl:for-each select="tokenize($splitNewName,'\|')">
                                <xsl:if test="not(.='')">
                                    <xsl:value-of select="concat(.,' ')"/>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:variable>
                        <tr>
                            <td>
                                <xsl:value-of select="$nbsp"/>
                            </td>
                            <td>
                                <xsl:value-of select="$spellChkName"/>
                            </td>
                        </tr>
                    </xsl:for-each>
                    <tr>
                        <td>
                            <xsl:value-of select="$nbsp"/>
                        </td>
                        <td>
                            <b>Attribute names</b>
                        </td>
                    </tr>
                    <xsl:for-each
                        select="distinct-values($allAttributes/allAttributes//attribute/@name)">
                        <xsl:variable name="newname" select="."/>
                        <xsl:variable name="newnameValue"
                            select="if(contains($newname,':')) then substring-after($newname,':') else $newname"/>
                        <xsl:variable name="splitNewName">
                            <xsl:value-of
                                select="camed:splitNewWords($newnameValue,$newnameValue,'true')"/>
                        </xsl:variable>
                        <xsl:variable name="spellChkName">
                            <xsl:for-each select="tokenize($splitNewName,'\|')">
                                <xsl:if test="not(.='')">
                                    <xsl:value-of select="concat(.,' ')"/>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:variable>
                        <tr>
                            <td>
                                <xsl:value-of select="$nbsp"/>
                            </td>
                            <td>
                                <xsl:value-of select="$spellChkName"/>
                            </td>
                        </tr>
                    </xsl:for-each>
                </table>
                <p>
                    <hr/>
                </p>
            </body>
        </html>

    </xsl:template>

    <xsl:template name="checkRules">
        <xsl:param name="rules"/>
        <!-- Conflicting rules check -->
        <xsl:for-each select="$rules">
            <xsl:for-each select="as:constraint">
                <xsl:variable name="action" select="substring-before(@action,'(')"/>
                <xsl:variable name="count"
                    select="count(../as:constraint[not(exists(@condition) or @context)]/@action[substring-before(.,'(')=$action])"/>
                <xsl:if test="$count>1 and(not(contains($action,'datatype')))">
                    <xsl:choose>
                        <xsl:when test="contains($action,'restrictValues')">
                            <li>Caution: multiple rule values [<xsl:value-of select="$action"/>] on
                                    <xsl:value-of select="@xpath"/></li>
                        </xsl:when>
                        <xsl:otherwise>
                            <li style="color:red">Duplicate rule action [<xsl:value-of
                                    select="$action"/>] on <xsl:value-of select="@xpath"/></li>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
                <xsl:if
                    test="$action='makeMandatory' and(count(../as:constraint[contains(@action,'makeOptional')]))">
                    <li style="color:red">Conflicting rules <xsl:value-of select="$action"/>,
                        makeOptional on <xsl:value-of select="@xpath"/></li>
                </xsl:if>
            </xsl:for-each>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="checkForChildCode">
        <xsl:param name="Parent"/>
        <xsl:variable name="findAttr" select="concat($Parent,'/@')"/>
        <xsl:variable name="attributes"
            select="if($findAttr='/@') then '' else $rules//as:Attribute[contains(@path,$findAttr)]"/>
        <xsl:variable name="elements"
            select="if($Parent='') then '' else $rules//as:Element[contains(@path,$Parent)]"/>
        <xsl:variable name="attribRestriction">
            <xsl:choose>
                <xsl:when test="$attributes=''"/>
                <xsl:otherwise>
                    <xsl:for-each select="$attributes">
                        <xsl:for-each select="as:Rule/as:constraint">
                            <xsl:choose>
                                <xsl:when test="contains(./@xpath,'*')"/>
                                <!-- ignore choiceItem() rules -->
                                <xsl:when test="contains(@action,'Mask(')"/>
                                <!-- set Mask content restriction applies -->
                                <xsl:when test="starts-with(@action,'make')"/>
                                <!-- ignore cardinality rules -->
                                <xsl:when test="starts-with(@action,'exclude')"/>
                                <!-- ignore cardinality rules -->
                                <xsl:when test="starts-with(@action,'set')"/>
                                <!-- ignore value rules -->
                                <xsl:when test="starts-with(@action,'datatype')"/>
                                <!-- ignore content rules -->
                                <xsl:when test="contains(@action,'restrictValues')"><xsl:value-of
                                        select="./@xpath"/>|</xsl:when>
                                <xsl:otherwise/>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="elemRestriction">
            <xsl:choose>
                <xsl:when test="$elements=''"/>
                <xsl:otherwise>
                    <xsl:for-each select="$elements">
                        <xsl:choose>
                            <xsl:when test="ends-with(@path,$Parent)"/>
                            <xsl:otherwise>
                                <xsl:for-each select="as:Rule/as:constraint">
                                    <xsl:choose>
                                        <xsl:when test="contains(./@xpath,'*')"/>
                                        <!-- ignore choiceItem() rules -->
                                        <xsl:when test="contains(@action,'Mask')"/>
                                        <!-- set Mask content restriction applies -->
                                        <xsl:when test="starts-with(@action,'make')"/>
                                        <!-- ignore cardinality rules -->
                                        <xsl:when test="starts-with(@action,'exclude')"/>
                                        <!-- ignore cardinality rules -->
                                        <xsl:when test="starts-with(@action,'set')"/>
                                        <!-- ignore value rules -->
                                        <xsl:when test="starts-with(@action,'datatype')"/>
                                        <!-- ignore content rules -->
                                        <xsl:when test="contains(@action,'restrictValues')"
                                                ><xsl:value-of select="./@xpath"/>|</xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="attrResults" select="distinct-values(tokenize($attribRestriction,'\|'))"/>
        <xsl:variable name="elemResults" select="distinct-values(tokenize($elemRestriction,'\|'))"/>
        <xsl:variable name="childNodes">
            <xsl:value-of select="concat('Parent: ',$Parent,'|')"/>
            <xsl:choose>
                <xsl:when test="count($attrResults) &gt; 0">
                    <xsl:for-each select="$attrResults">
                        <xsl:choose>
                            <xsl:when test="string-length(.) &lt; 1"/>
                            <xsl:when test="contains(upper-case(.),'VALUE')">- matched:
                                    <xsl:value-of select="concat('@',substring-after(.,'/@'))"
                                />|</xsl:when>
                            <xsl:otherwise> - attribute: <xsl:value-of
                                    select="concat('@',substring-after(.,'/@'))"/>|</xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:when>
                <xsl:when test="count($elemResults) &gt; 0">
                    <xsl:for-each select="$elemResults">
                        <xsl:choose>
                            <xsl:when test="string-length(.) &lt; 1"/>
                            <xsl:otherwise>- child node: <xsl:value-of select="."/>|</xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$childNodes"/>
    </xsl:template>


    <xsl:template name="checkForChildDate">
        <xsl:param name="Parent"/>
        <xsl:variable name="findAttr" select="concat($Parent,'/@')"/>
        <xsl:variable name="attributes"
            select="if($findAttr='/@') then '' else $rules//as:Attribute[contains(@path,$findAttr)]"/>
        <xsl:variable name="elements"
            select="if($Parent='') then '' else $rules//as:Element[contains(@path,$Parent)]"/>
        <xsl:variable name="attribRestriction">
            <xsl:choose>
                <xsl:when test="$attributes=''"/>
                <xsl:otherwise>
                    <xsl:for-each select="$attributes">
                        <xsl:for-each select="as:Rule/as:constraint">
                            <xsl:choose>
                                <xsl:when test="contains(@action,'setDateMask')"><xsl:value-of select="./@xpath"/>|</xsl:when>
                                <xsl:otherwise/>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="elemRestriction">
            <xsl:choose>
                <xsl:when test="$elements=''"/>
                <xsl:otherwise>
                    <xsl:for-each select="$elements">
                        <xsl:choose>
                            <xsl:when test="ends-with(@path,$Parent)"/>
                            <xsl:otherwise>
                                <xsl:for-each select="as:Rule/as:constraint">
                                    <xsl:choose>
                                        <xsl:when test="contains(@action,'setDateMask')"><xsl:value-of select="./@xpath"/>|</xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="attrResults" select="distinct-values(tokenize($attribRestriction,'\|'))"/>
        <xsl:variable name="elemResults" select="distinct-values(tokenize($elemRestriction,'\|'))"/>
        <xsl:variable name="childNodes">
            <xsl:value-of select="concat('Parent: ',$Parent,'|')"/>
            <xsl:choose>
                <xsl:when test="count($attrResults) &gt; 0">
                    <xsl:for-each select="$attrResults">
                        <xsl:choose>
                            <xsl:when test="string-length(.) &lt; 1"/>
                            <xsl:when test="contains(upper-case(.),'DATE')">- matched: <xsl:value-of
                                    select="concat('@',substring-after(.,'/@'))"/>|</xsl:when>
                            <xsl:when test="contains(upper-case(.),'TIME')">- matched: <xsl:value-of
                                    select="concat('@',substring-after(.,'/@'))"/>|</xsl:when>
                            <xsl:otherwise> - attribute: <xsl:value-of
                                    select="concat('@',substring-after(.,'/@'))"/>|</xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:when>
                <xsl:when test="count($elemResults) &gt; 0">
                    <xsl:for-each select="$elemResults">
                        <xsl:choose>
                            <xsl:when test="string-length(.) &lt; 1"/>
                            <xsl:otherwise>- child node: <xsl:value-of select="."/>|</xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="$childNodes"/>
    </xsl:template>

    <xsl:template name="checkNDRrules">
        <xsl:param name="rulesChk"/>
        <!-- xsl:message>Rules[<xsl:copy-of select="$rulesChk"></xsl:copy-of>]</xsl:message -->
        <xsl:variable name="NDRresults">
            <xsl:for-each select="$rulesChk">
                <xsl:for-each select="as:constraint">
                    <xsl:variable name="chkPrefix">
                        <xsl:choose>
                            <xsl:when test="contains(@xpath,'@')"><xsl:value-of
                                    select="substring-before(substring-after(@xpath,'@'),':')"
                                /></xsl:when>
                            <xsl:otherwise><xsl:variable name="path" select="tokenize(@xpath,'/')"
                                    /><xsl:value-of
                                    select="substring-before($path[count($path)],':')"
                                /></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:if test="cam2func:checkDomain($chkPrefix)='true'">
                        <xsl:choose>
                            <!-- NIEM NDR rules: 1.1_rule_6-7, 1.1_rule_6-8, 1.3_rule_6-10, 1.3_rule_6-11, 1.3_rule_6-35, 1.3_rule_6-37, 1.3_rule_6-39, 1.3_rule_6-41, 1.3_rule_6-46, 1.3_rule_6-47, 1.3_rule_6-49, 1.3_rule_9-5, 1.3_rule_9-6, 1.3_rule_9-19, 1.3_rule_9-22 -->
                            <xsl:when test="contains(@action,'anyType')"><xsl:value-of
                                    select="concat('anyType:-',@action,'[',@xpath,']')"
                                />|</xsl:when>
                            <xsl:when test="contains(@action,'anySimpleType')"><xsl:value-of
                                    select="concat('anySimpleType:-',@action,'[',@xpath,']')"
                                />|</xsl:when>
                            <xsl:when test="contains(@action,'anyAttribute')"><xsl:value-of
                                    select="concat('anyAttribute:-',@action,'[',@xpath,']')"
                                />|</xsl:when>
                            <xsl:when test="contains(@action,'datatype(any)')"><xsl:value-of
                                    select="concat('any:-',@action,'[',@xpath,']')"/>|</xsl:when>
                            <xsl:when test="contains(@action,'setChoice')"><!-- Validate that the expression is correctly entered -->
                                <xsl:choose>
                                    <xsl:when test="not(contains(@action,'/*'))"><xsl:value-of
                                            select="concat('choice:-',@action,'[',@xpath,']')"
                                        />|</xsl:when><!-- Not applied to children so flag -->
                                    <xsl:when test="not(contains(@action,'['))"/><!-- No list of items so ignore -->
                                    <xsl:when test="contains(@action,'[')">
                                        <xsl:choose>
                                            <xsl:when test="contains(@action,' or ')"/><!-- Ignore when has multiple conditionals -->
                                            <xsl:otherwise><xsl:value-of
                                                  select="concat('choice:-',@action,'[',@xpath,']')"
                                                />|</xsl:otherwise><!-- Flag when singleton conditional -->
                                        </xsl:choose>
                                    </xsl:when><!-- List of items so ignore -->
                                    <xsl:otherwise/><!-- Ignore -->
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:if>
                </xsl:for-each>
            </xsl:for-each>
            <!-- Now check naming rules -->
            <xsl:variable name="attrAnon">
                <xsl:for-each select="$rules//as:Attribute">
                    <xsl:if
                        test="count(camed:annotation/camed:documentation[@type='Definition']) = 0"
                            ><xsl:variable name="prefix" select="substring-before(@name,':')"
                            /><xsl:if test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                select="@name"/>|</xsl:if></xsl:if>
                </xsl:for-each>
            </xsl:variable>noAnnoAttributes:-<xsl:value-of
                select="count(distinct-values(tokenize($attrAnon,'\|')))"/><xsl:value-of
                select="'\'"/>badAnnoAttributes:-<xsl:value-of select="$attrAnon"/><xsl:value-of
                select="'\'"/>
            <xsl:variable name="attrNames">
                <xsl:for-each select="$rules//as:Attribute">
                    <xsl:choose>
                        <xsl:when test="$camelCaseMode='NIEM'"><xsl:if
                                test="not(matches(if(contains(@name,':')) then substring(substring-after(@name,':'),1,1) else substring(@name,1,1),'[a-z]'))"
                                    ><xsl:variable name="prefix"
                                    select="substring-before(@name,':')"/><xsl:if
                                    test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                        select="@name"/>|</xsl:if></xsl:if></xsl:when>
                        <xsl:when test="$camelCaseMode='LowerCase'"><xsl:if
                                test="not(matches(if(contains(@name,':')) then substring(substring-after(@name,':'),1,1) else substring(@name,1,1),'[a-z]'))"
                                    ><xsl:variable name="prefix"
                                    select="substring-before(@name,':')"/><xsl:if
                                    test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                        select="@name"/>|</xsl:if></xsl:if></xsl:when>
                        <xsl:otherwise><xsl:if
                                test="not(matches(if(contains(@name,':')) then substring(substring-after(@name,':'),1,1) else substring(@name,1,1),'[A-Z]'))"
                                    ><xsl:variable name="prefix"
                                    select="substring-before(@name,':')"/><xsl:if
                                    test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                        select="@name"/>|</xsl:if></xsl:if></xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
            </xsl:variable>attributenames:-<xsl:value-of
                select="count(distinct-values(tokenize($attrNames,'\|')))"/><xsl:value-of
                select="'\'"/>badLowerAttributes:-<xsl:value-of select="$attrNames"/><xsl:value-of
                select="'\'"/>
            <xsl:variable name="attrNamesChar">
                <xsl:for-each select="$rules//as:Attribute">
                    <xsl:variable name="chkName"
                        select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                    <xsl:if test="contains($chkName,'_') or(contains($chkName,'.'))"><xsl:variable
                            name="prefix" select="substring-before(@name,':')"/><xsl:if
                            test="cam2func:checkDomain($prefix)='true'"><xsl:value-of select="@name"
                            />|</xsl:if></xsl:if>
                </xsl:for-each>
            </xsl:variable>attributenamechar:-<xsl:value-of
                select="count(distinct-values(tokenize($attrNamesChar,'\|')))"/><xsl:value-of
                select="'\'"/>badCharAttributes:-<xsl:value-of select="$attrNamesChar"
                /><xsl:value-of select="'\'"/>
            <xsl:variable name="elemAnon">
                <xsl:for-each select="$rules//as:Element">
                    <xsl:if
                        test="count(camed:annotation/camed:documentation[@type='Definition']) = 0"
                            ><xsl:variable name="prefix" select="substring-before(@name,':')"
                            /><xsl:if test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                select="@name"/>|</xsl:if></xsl:if>
                </xsl:for-each>
            </xsl:variable>elements:-<xsl:value-of
                select="count(distinct-values(tokenize($elemAnon,'\|')))"/><xsl:value-of
                select="'\'"/>badAnnoElements:-<xsl:value-of select="$elemAnon"/><xsl:value-of
                select="'\'"/>
            <xsl:variable name="elemNames">
                <xsl:for-each select="$rules//as:Element">
                    <xsl:choose>
                        <xsl:when test="$camelCaseMode='LowerCase'"><xsl:if
                                test="not(matches(if(contains(@name,':')) then substring(substring-after(@name,':'),1,1) else substring(@name,1,1),'[a-z]'))"
                                    ><xsl:variable name="prefix"
                                    select="substring-before(@name,':')"/><xsl:if
                                    test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                        select="@name"/>|</xsl:if></xsl:if></xsl:when>
                        <xsl:otherwise><xsl:if
                                test="not(matches(if(contains(@name,':')) then substring(substring-after(@name,':'),1,1) else substring(@name,1,1),'[A-Z]'))"
                                    ><xsl:variable name="prefix"
                                    select="substring-before(@name,':')"/><xsl:if
                                    test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                        select="@name"/>|</xsl:if></xsl:if></xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
            </xsl:variable>elementnames:-<xsl:value-of
                select="count(distinct-values(tokenize($elemNames,'\|')))"/><xsl:value-of
                select="'\'"/>badFirstelements:-<xsl:value-of select="$elemNames"/><xsl:value-of
                select="'\'"/>
            <xsl:variable name="elemNamesChar">
                <xsl:for-each select="$rules//as:Element">
                    <xsl:variable name="chkName"
                        select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                    <xsl:if test="contains($chkName,'_') or(contains($chkName,'.'))"><xsl:variable
                            name="prefix" select="substring-before(@name,':')"/><xsl:if
                            test="cam2func:checkDomain($prefix)='true'"><xsl:value-of select="@name"
                            />|</xsl:if></xsl:if>
                </xsl:for-each>
            </xsl:variable>elementnamechar:-<xsl:value-of
                select="count(distinct-values(tokenize($elemNamesChar,'\|')))"/><xsl:value-of
                select="'\'"/>badCharElements:-<xsl:value-of select="$elemNamesChar"/><xsl:value-of
                select="'\'"/>
            <!-- Child and Attribute Names contain representative terms -->
            <xsl:variable name="attrNamesRepr">
                <xsl:for-each select="$rules//as:Attribute">
                    <xsl:variable name="chkName"
                        select="if(contains($namespaceDetail,'structure|')) then '' else cam2func:has-representation-term(if(contains(@name,':')) then substring-after(@name,':') else @name)"/>
                    <xsl:if test="string-length($chkName) &gt; 0"><xsl:variable name="prefix"
                            select="substring-before(@name,':')"/><xsl:if
                            test="cam2func:checkDomain($prefix)='true'"><xsl:value-of select="@name"
                            />|</xsl:if></xsl:if>
                </xsl:for-each>
            </xsl:variable>attributenamerepr:-<xsl:value-of
                select="count(distinct-values(tokenize($attrNamesRepr,'\|')))"/><xsl:value-of
                select="'\'"/>badattributes:-<xsl:value-of select="$attrNamesRepr"/><xsl:value-of
                select="'\'"/>
            <xsl:variable name="elemNamesRepr">
                <xsl:for-each select="$rules//as:Element">
                    <xsl:variable name="chkName" select="cam2func:has-representation-term(if(contains(@name,':')) then substring-after(@name,':') else @name)"/>
                    <xsl:if test="string-length($chkName) &gt; 0">
                        <xsl:choose>
                            <xsl:when test="count(as:Element) &gt; 0"/>
                            <xsl:otherwise><xsl:variable name="prefix"
                                    select="substring-before(@name,':')"/><xsl:if
                                    test="cam2func:checkDomain($prefix)='true'"><xsl:value-of
                                        select="@name"/>|</xsl:if></xsl:otherwise>
                        </xsl:choose>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>elementnamerepr:-<xsl:value-of
                select="count(distinct-values(tokenize($elemNamesRepr,'\|')))"/><xsl:value-of
                select="'\'"/>badelements:-<xsl:value-of select="$elemNamesRepr"/><xsl:value-of
                select="'\'"/>
        </xsl:variable>
        <xsl:value-of select="$NDRresults"/>
    </xsl:template>

    <xsl:template match="*" mode="names">
        <xsl:for-each select=".//element/@name">
            <xsl:value-of
                select="if(contains(.,':')) then  substring(substring-after(.,':'),1,1) else substring(.,1,1)"
            />
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match="*" mode="mixedContent">
        <xsl:if test="./as:Element"><!-- if child elements of parent, check parent for content rules -->
            <xsl:if test="./as:Rule/as:constraint[contains(@action,'restrictValues')] or(./as:Rule/as:constraint[contains(@action,'datatype')] ) or(./as:Rule/as:constraint[contains(@action,'Mask(')]) or(./as:Rule/as:constraint[contains(@action,'setNumberRange')]) or(./as:Rule/as:constraint[contains(@action,'setLength')])">
                <xsl:variable name="chkDataType" select="./as:Rule/as:constraint[contains(@action,'datatype')]/@action"/>
                <xsl:if test="count($chkDataType) &gt; 1 or(not(contains($chkDataType[1],'anyAttribute')))">
                <li> <xsl:value-of select="replace(./as:Rule/as:constraint[1]/@xpath,'//','/')"/> <xsl:for-each select="./as:Rule/as:constraint[contains(@action,'restrictValues')]/@action | ./as:Rule/as:constraint[contains(@action,'datatype')]/@action |./as:Rule/as:constraint[contains(@action,'Mask(')]/@action |./as:Rule/as:constraint[contains(@action,'setNumberRange')]/@action |./as:Rule/as:constraint[contains(@action,'setLength')]/@action"> - <xsl:value-of select="substring-before(substring-after(.,'('),')')"></xsl:value-of></xsl:for-each></li>
                </xsl:if>
            </xsl:if>
        </xsl:if>
    </xsl:template>

    <!-- Start shared templates from CXF-rename-util.xsl -->

    <xsl:template match="as:Element" mode="list">
        <xsl:param name="type"/>
        <xsl:variable name="targetNS"/>
        <xsl:variable name="thisNS" select="substring-before(@name,':')"/>
        <xsl:if
            test="$type='element' and($targetNS='' or($targetNS='*') or($targetNS=$thisNS)) or(starts-with($targetNS,'~') and not($targetNS=concat('~',$thisNS)))">
            <xsl:element name="element">
                <xsl:attribute name="name" select="@name"/>
            </xsl:element>
        </xsl:if>
        <xsl:if test="$type='attribute'">
            <xsl:apply-templates mode="list" select="as:Attribute">
                <xsl:with-param name="type" select="$type"/>
            </xsl:apply-templates>
        </xsl:if>
        <xsl:apply-templates mode="list" select="as:Element">
            <xsl:with-param name="type" select="$type"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="as:Attribute" mode="list">
        <xsl:param name="type"/>
        <xsl:variable name="targetNS"/>
        <xsl:variable name="thisNS" select="substring-before(@name,':')"/>
        <xsl:if
            test="$type='attribute' and($targetNS='' or($targetNS='*') or($targetNS=$thisNS)) or(starts-with($targetNS,'~') and not($targetNS=concat('~',$thisNS)))">
            <xsl:element name="attribute">
                <xsl:attribute name="name" select="@name"/>
            </xsl:element>
        </xsl:if>
    </xsl:template>

    <xsl:template match="node()" mode="list"/>

    <!-- End shared templates from CXF-rename-util.xsl -->

</xsl:stylesheet>
